// #####################################################################################
// #                                                                                   #
// #     Headerfile der Klasse Drawing:                                                #
// #                                                                                   #
// #     Programmierer:     Sascha Kurz                                                #
// #     Kontakt:           KurzTEF@web.de					       #
// #     Aktualisierungen:  http://www.stud.uni-bayreuth.de/~a8581/                    #
// #                                                                                   #
// #     getestete Betriebssysteme:       Suse Linux				       #
// #					  DOS 					       #	
// #     Compiler:                        g++ 2.95.2                                   #
// #                                                                                   #
// #     letzte Aenderung:                08.07.2001                                   #
// #										       #
// #####################################################################################



// Anwendungen:
// ------------
// Mit Hilfe der Klasse koennen die Anzahl der Polygone eines regelmaessigen
// n-Ecks berechnet werden.
// Beispiel:
// Das regelmaessige 4-Eck:
//
//           |-------|
//           |\  1  /|      hat 4 Dreiecke.
//           | \   / |      hier mit 1,2,3, und 4 bezeichnet.
//           |4 \/   |      Das aeussere 4-Eck wird nicht gezaehlt, da es
//           |  /\ 2 |      einen Punkt im inneren enthaelt
//           | /  \  |
//           |/  3 \ |
//           |-------|
// Eine kleine Beispieltabelle:
// Anzahl der Knoten | 3-Ecke | 4-Ecke | 5-Ecke | 6-Ecke | 7-Ecke | 8-Ecke | Polygone
// ----------------------------------------------------------------------------------
//       	3     	    1                                                      1
//	     	4           4        0                                             4
//		5          10        0        1                                   11
//		6          18        6        0        0                          19
//		7          35        7        7        0        1                 50
//		8          56       24        0        0        0        0        80
//		9          90       36       18        9        0        0       154
//	       10         120       90       10        0        0        0       220
//	       11         176      132       44       22        0        0       375
//	       12         276      168        0        0        0        0       444
//	       13         377      234      117       39        0       13       781
//	       14         476      378       98        0        0        0       952
//	       15         585      600      150      105       15        0      1456
//	       16         848      672      128       48        0        0      1696
//	       17       1.054      901      357      136       17       34      2500
		









#define PI       3.141592653589793238462643383		// Kreiskonstante Pi
#define FASTNULL 0.0000000000001			// Diese Konstante wird wegen der begrenzten Rechengenauikeit
							// benoetigt. Alle Werte, die von absolutem Betrag her kleiner
							// sind, werden mit 0 identifiziert.	
#define TRICK 1						// Standardwert 0, 1 => Es werden abkuerzende Tricks verwendet, di natuerlich zum Teil die Tests unterlaufen							
							
						
template<class T,class S> class Drawing;		// Vorwaertsdeklaration der Klasse Drawing

int SpeicherFehler(void);				// Standardfehlerausgabe fuer Speicherfehler	
template<class T> int swap(T&,T&);			// Allgemien Swap-Routine
template<class T> T min(T,T);				// Allgemeine Minimumsfunktion



// Bemerkung zu der Template-Klasse
// T soll ein Ganzzahldatentyp sein
// S soll ein Gleitkommadatentyp sein
// Genauere Informationen stehen in der Dokumentation
template<class T,class S> class Drawing			// Klassendefinition von Drawing
{
    private:
    int Bedingung(T,T);					// interne Methode, die ueberprueft ob sich zwei Geraden in einem Sektor eines regulaeren n-Ecks schneiden koennen, wird von BerechneKreuzungspunkte_symmetrisch verwendet
    int Bedingung2(T,T);				// interne Methode, die ueberpruft, ob eine Gerade ueberhaupt durch einen Sektor eines regelmaessigen n-Ecks gehen kann
    int InnererPunkt(T*,int,T);				// interne Methode, die ueberprueft, ob es einen Punkt innerhalb eines Polygons gibt
    int delta(int,int);					// interne Methode, mathematische Berechnung aus "The number of intersection points made by the diagonals of a regular polygon" [1]
    int BestimmeNachbarnDerKnoten(void);		// interne Methode, wird von BerechneNachbarArrays verwendet
    int BestimmeNachbarn_symmetrisch(void);		// interne Methode, wird von BerechneNachbarArrays_symmetrisch verwendet
    int BestimmeNachbarnDerKreuzungspunkte(void);	// interne Methode, wird von BestimmeNachbarArrays verwendet
    int preInitialisiereNachbararrays(void);		// interne Methode, wird von BerechneNachbararrays verwendet
    int spareSpeicher_1(void);				// brutales sparen von Speicherplatz
    int spareSpeicher_2(void);				// brutales sparen von Speicherplatz
    int spareSpeicher_3(void);				// brutales sparen von Speicherplatz
    int spareSpeicher_4(void);				// brutales sparen von Speicherplatz
    int spareSpeicher_5(void);				// brutales sparen von Speicherplatz
    int QS_x(T*,T,T);					// Quicksortvariante zum Sortieren von Geraden nach der X-Koordinate
    int QS_y(T*,T,T);					// Quicksortvariante zum Sortieren von Geraden nach der Y-Koordinate
    int MS_x(T*,T,T);					// Mergesortvariante zum Sortieren von Geraden nach der X-Koordinate
    int MS_y(T*,T,T);					// Mergesortvariante zum Sortieren von Geraden nach der Y-Koordinate
    int HS_x(T*,T);			    		// Heapsortvariante zum Sortieren von Geraden nach der X-Koordinate
    int HS_y(T*,T);					// Heapsortvariante zum Sortieren von Geraden nach der Y-Koordinate
    T AnzahlDerKreuzungspunkte;				// speichert die Anzahl der Kreuzungspunkte
    int AnzahlDerKnoten;				// speichert die Anzahl der Knoten
    T AnzahlDerKanten; 		                        // speichert die Anzahl der Kanten
    T AnzahlDerFlaechen;                                // speichert die Anzahl der Flaechen
    S* KnotenX; 	                  		// dynamisches Array zur Speicherung der X-Koordinaten der Knoten
    S* KnotenY; 	                                // dynamisches Array zur Speicherung der Y-Koordinaten der Knoten
    S* KreuzungspunktX;					// dynamisches Array zur Speicherung der X-Koordinaten der Kreuzungspunkte
    S* KreuzungspunktY;					// dynamisches Array zur Speicherung der Y-Koordinaten der Kreuzungspunkte
    int** Matrix;					// dynamisches Array zur Speicherung der Verbindungsmatrix des Graphen
    T* Nachbar1;					// dynamisches Array zur Speicherung der Nachbarschaftsliste des Kreuzungsgraphen
    T* Nachbar2;					// dynamisches Array um auf Nachbar1 zugreifen zu koennen
    T* Strecke1;                                        // dynamisches Array zur Speicherung der Anfangsknoten der Strecken
    T* Strecke2;                                        // dynamisches Array zur Speicherung der Endknoten der Strecken
    T** GeradenNr;					// dynamisches Array zur Speicherng der Geradennummer zu zwei beliebigen Knoten
    T* Gerade1;						// dynamisches Array zur Speicherung der Kreuzungspunkte auf den Geraden
    T* Gerade2;						// dynamisches Array um auf Gerade1 zugreifen zu koennen
    int* AnzahlDerNachbarn;				// dynamisches Array zur Speicherung des Knotengrades der Kreuzungspunkte
    int* Knotengrad;					// dynamisches Array zur Speicherung des Knotengrades der Knoten
    int init(int);					// Initialisierungsfunktion
    T BerechneZaehler(T);				// interne Methode, die einen Zaehler berechnet
    T BerechneZaehler_s(T);				// interne Methode, die einen Zaehler berechnet
    int setIKnotenX(T,S);				// interne Methode, welche die X-Koordinate eines Knotens oder Kreuzungspunktes setzt
    int setIKnotenY(T,S);                               // interne Methode, welche die Y-Koordinate eines Knotens oder Kreuzungspunktes setzt
    S IKnotenX(T);		    			// gibt die X-Koordinate eines Knotens oder Kreuzungspunktes zurueck
    S IKnotenY(T);					// gibt die Y-Koordinate eines Knotens oder Kreuzungspunktes zurueck
    S IKreuzungspunktX(T);				// gibt die X-Koordinate eines Knotens oder Kreuzungspunktes zurueck
    S IKreuzungspunktY(T);				// gibt die Y-Koordinate eines Knotens oder Kreuzungspunktes zurueck
    int IKnotengrad(T);				        // gibt den Knotengrad eines Knotens oder Kreuzungspunktes zurueck
    int IAnzahlDerNachbarn(T);	     			// gibt den Knotengrad eines Knotens oder Krezungspunktes zurueck
    T Nachbar(T,T);	     				// gibt u einem Knoten oder Kreuzungspunkt den j-ten Nachbarn zurueck
    S BerechneWinkel(T,T,T);				// Berechnet den Winkel zwischen 3 Knoten bzw. Kreuzungspunkten
    int TestAeussereKante(T,T,T,T);			// Testet, ob die uebergebenen Knoten eine auessere Kante sind
    int Versuch(T,T,T,T,int);				// interne Methode, die fuer die Berechnung der Anzahl der Polygone benoetigt wird
    int Versuch_o(T,T,T,T,int);				// interne Methode, die fuer die Berechnung der Anzahl der Polygone benoetigt wird
    T* AnzahlDerPolygone;				// Speichert die Anzahl der 3-Ecke, 4-Ecke, ...	
    int Schnittpunktfinden(S,S,S,S,S,S,S,S,S&,S&);	// Versucht zu 4 Punkten den Schnittpunkt zu bestimmen
  public:
    Drawing();						// Standardkonstruktor
    Drawing(int);					// Konstruktor der die Anzahl der Knoten uebergeben bekommt
    ~Drawing();						// Destruktor

    // Methoden zum Erzeugen
    int CreateCompleteGraph(void);      		// erzeugt eines vollstaendigen Graphen
    int CreateRegularPoints(void);                      // erzeuge die Koordinaten eines regelmaessigen n-Ecks
    int CreateRegularNGon(void);                        // erzeuge regelmaessiges n-Eck

    // set-Methoden
    int setMatrix(int,int,int);				// Belege die Position i,j der Verbindungs-Matrix mit einem Wert
    int setMatrix(int,int*);				// Belege die i-te Spalte der Verbindungs-Matrix mit einem Werte-Vektor	
    int setMatrix(int**);				// Belege die Verbindungs-Matrix mit einer uebergebenen Matrix
    int setKnotenX(T,S);				// Belege die X-Koordinate eines Knotens
    int setKnotenY(T,S);				// Belege die Y-Koordinate eines Knotens

    // get-Methoden
    int getAnzahlDerKnoten(void);        		// gibt die Anzahl der Knoten zurueck	
    T getAnzahlDerKanten(void);                         // gibt die Anzahl der Kanten zurueck
    T getAnzahlDerKreuzungspunkte(void);	        // gibt die Anzahl der Kreuzungspunkte zurueck
    S getKnotenX(int);                                  // gibt die X-Koordinate eines Knotens zurueck
    S getKnotenY(int);  	                        // gibt die Y-Koordinate eines Knotens zurueck
    S getKreuzungspunktX(T);				// gibt die X-Koordinate eines Kreuzungspunktes zurueck	
    S getKreuzungspunktY(T);                            // gibt die Y-Koordinate eines Kreuzungspunktes zurueck
    int getKnotengrad(int);    				// gibt den Knotengrad eines Knotens zurueck
    int getAnzahlDerNachbarn(T);                        // gibt die Anzahl der Nachbarn eines Knotens zurueck
    int getMatrix(int,int);                             // gibt den Wert an einer Position der Verbindungsmatrix zurueck
    T getPolygone(int);					// gibt die Anzahl der Polygone zurueck


    // Methoden zum zusammenbauen von eigenen Berechnungen
    // Baukasten

      // Schritt 1:
    	int BerechneKreuzungspunkte(void);		// Berechnet die Kreuzungspunkte
   	int BerechneKreuzungspunkte_symmetrisch(void);	// Berechnet die Kreuzungspunkte eines Sektors eines regelmaeigen n-Ecks

      // Schritt 2:	
    	int sortiereGeradenQS(void);			// sortiert die Geraden mit Quicksort	
    	int sortiereGeradenMS(void);                    // sortiert die Geraden mit Mergesort
    	int sortiereGeradenHS(void);                    // sortiert die Geraden mit Heapsort
    	
      // Schritt 2,5:   (ist nur bei einer Geraden Anzahl von Ecken notwendig) 	
	int VerschmelzeMehrfachSchnittpunkte(void);		// interne Methode, sorgt dafuer, dass es bei einem regelmaessigen n-Eck auch Schnittpunkte aus mehr als zwei Geraden geben kann    	

      // Schritt 3:	
    	int BerechneNachbarArrays(void);		// Berechnet die Nachbar-Arrays
    	int BerechneNachbarArrays_o1(void);		// Berechnet die Nachbar-Arrays, Speicherplatz optimierte Variante 1, vor der Benutzung unbedingt Bemerkung um Quellcode durchlesen
    	int BerechneNachbarArrays_o2(void);		// Berechnet die Nachbar-Arrays, Speicherplatz optimierte Variante 2, vor der Benutzung unbedingt Bemerkung im Quellcode durchlesen
    	int BerechneNachbarArrays_symmetrisch(void);	// Berechnet die Nachbar-Arrays, zur symmetrischen Berechnung
    	int BerechneNachbarArrays_o1_symmetrisch(void);	// Berechnet die Nachbar-Arrays, Speicherplatz optimierte Variante 1, vor der Benutzung unbedingt Bemerkung um Quellcode durchlesen, zur symmetrischen Berechnung
    	int BerechneNachbarArrays_o2_symmetrisch(void);	// Berechnet die Nachbar-Arrays, Speicherplatz optimierte Variante 2, vor der Benutzung unbedingt Bemerkung im Quellcode durchlesen, zur symmetrischen Berechnung
      // Schritt 4:
    	T BerechneAnzahlDreiecke(T,T);                  // Berechnet die Anzahl der Dreiecke, hierbei muss beachtet werden, dass kein Test auf innere Punkte erfolgt, vor der Benutzung unbedingt Bemerkung im Quellcode durchlesen
    	T BerechneAnzahlDreiecke_allgemein(T,T);	// Berechnet die Anzahl der Dreiecke, allgemein gueltige Methode	
    	T BerechneAnzahlVierecke(T,T);                  // Berechnet die Anzahl der Dreiecke und Vierecke mit einer langsamen allgemeinen Methode
    	T BerechneAnzahlFuenfecke(T,T);			// Berechnet die Anzahl der Dreiecke, Vierecke und Fuenfecke mit einer langsamen allgemeinen Methode
    	T BerechneAnzahlDreiecke_o(void);               // Berechnet die Anzahl der Dreiecke, Speicherplatz optimierte Variante, vor der Benutzung unbedingt Bemerkung im Quellcode durchlesen
    	T BerechneAnzahlDreiecke_o_allgemein(void);     // Berechnet die Anzahl der Dreiecke, Speicherplatz optimierte Variante, vor der Benutzung unbedingt Bemerkung im Quellcode durchlesen, allgemeingueltig
    	T BerechneAnzahlDreiecke_o_symmetrisch(void);	// Berechnet die Anzahl der Dreicke des n-ten Teils eines regelmaessigen n-Ecks	
    	T BerechneAnzahlDreiecke_symmetrisch(void);   	// Berechnet die Anzahl der Dreicke des n-ten Teils eines regelmaessigen n-Ecks, Speicherplatzoptimierte Version, vor der Benutzung unbedingt Bemerkung im Quellcode durchlesen	
    	T BerechneAnzahlPolygone(void);			// Berechnet die Anzahl der Polygone
    	T BerechneAnzahlPolygone_o(void);		// Berechnet die Anzahl der Polygone
    // Ende des Baukastens	

    // kleine "Beispielprogramme"
    // sollten in der Regel benutzt werden
    T AnzahlDreieckeRegularPolygon1(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!), Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon2(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!), nicht Speicherplatz optimierte Version
    T AnzahlPolygoneRegularPolygon1(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken (n ungerade!), Speicherplatz optimierte Version
    T AnzahlPolygoneRegularPolygon2(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken (n ungerade!), nicht Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon3(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!), Speicherplatz optimierte Version, gibt die Rechenzeiten aus
    T AnzahlDreieckeRegularPolygon4(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!), nicht Speicherplatz optimierte Version, gibt Rechenzeiten aus
    T AnzahlPolygoneRegularPolygon3(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken (n ungerade!), Speicherplatz optimierte Version, gibt Rechenzeiten aus
    T AnzahlPolygoneRegularPolygon4(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken (n ungerade!), nicht Speicherplatz optimierte V, gibt Rechenzeiten aus
    T AnzahlDreieckeRegularPolygon5(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken, Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon6(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken, nicht Speicherplatz optimierte Version
    T AnzahlPolygoneRegularPolygon5(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken, Speicherplatz optimierte Version
    T AnzahlPolygoneRegularPolygon6(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken, nicht Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon7(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken, Speicherplatz optimierte Version, gibt die Rechenzeiten aus
    T AnzahlDreieckeRegularPolygon8(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken, nicht Speicherplatz optimierte Version, gibt Rechenzeiten aus
    T AnzahlPolygoneRegularPolygon7(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken, Speicherplatz optimierte Version, gibt Rechenzeiten aus
    T AnzahlPolygoneRegularPolygon8(int);		// Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken, nicht Speicherplatz optimierte V, gibt Rechenzeiten aus
    T AnzahlDreieckeRegularPolygon1_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!) (symmetrisch), Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon2_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!) (symetrisch), nicht Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon3_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!) (symmetrisch), Speicherplatz optimierte Version, gibt die Rechenzeiten aus
    T AnzahlDreieckeRegularPolygon4_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (n ungerade!) (symmetrisch), nicht Speicherplatz optimierte Version, gibt Rechenzeiten aus
    T AnzahlDreieckeRegularPolygon5_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch), Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon6_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch), nicht Speicherplatz optimierte Version
    T AnzahlDreieckeRegularPolygon7_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch), Speicherplatz optimierte Version, gibt die Rechenzeiten aus
    T AnzahlDreieckeRegularPolygon8_s(int);		// Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch), nicht Speicherplatz optimierte Version, gibt Rechenzeiten aus

    // Informationsmethoden
    // hier muss so gut wie nicht gerechnet werden
    T AnzahlRegionsRegularPolygon(int);			// Berechnet die Anzahl der Regionen (Flchen) eines regulaeren Polygons nach einer Formel aus "The number of intersection points made by the diagonals of a regular polygon" [1]
    T AnzahlIntersectionsRegularPolygon(int);		// Berechnet die Anzahl der Kreuzungspunkte (intersection points) eines regulaeren Polygons nach einer Formel aus "The number of intersection points made by the diagonals of a regular polygon" [1]
    T NachschlagenDreiecke(int);      			// gibt die Anzahl der Dreiecke eines regelmaessigen n-Ecks zurueck, die es in einem Standardfile sucht
    T NachschlagenDreiecke(int,char*);			// gibt die Anzahl der Dreiecke eines regelmaessigen n-Ecks zurueck, die es in einem file sucht
    T NachschlagenPolygone(int,int);			// gibt die Anzahl der m-Ecke eines regelmaessigen n-Ecks zurueck, die es in einem Standardfile sucht
    T NachschlagenPolygone(int,int,char*);		// gibt die Anzahl der m-Ecke eines regelmaessigen n-Ecks zurueck, die es in einem file sucht
};

// Realisierungsfile:	drawing.cpp
// Beispielprogramme:   bsp_draw_1.cpp
//			bsp_draw_2.cpp
//                      bsp_draw_3.cpp
//                      bsp_draw_4.cpp
//                      bsp_draw_5.cpp
//                      bsp_draw_5.cpp
//                      bsp_draw_6.cpp
//                      bsp_draw_7.cpp
//                      bsp_draw_8.cpp
//                      bsp_draw_9.cpp
//                      bsp_draw_10.cpp
//                      bsp_draw_11.cpp
//                      bsp_draw_12.cpp
// Dokumentation:	drawing.html
// Resultfiles:		result_dreiecke   ->  Standardfile zum Nachschlagen von Ergebnissen
//			result_polygone	  ->  Standardfile zum Nachschlagen von Ergebnissen
