// #####################################################################################
// #                                                                                   #
// #     Realisierungsfile der Klasse Drawing:                                         #
// #                                                                                   #
// #     Programmierer:     Sascha Kurz                                                #
// #     Kontakt:           KurzTEF@web.de					       #
// #     Aktualisierungen:  http://www.stud.uni-bayreuth.de/~a8581/                    #
// #                                                                                   #
// #     getestete Betriebssysteme:       Suse Linux				       #
// #					  DOS					       #	
// #     Compiler:                        g++ 2.95.2                                   #
// #     Command:	                  g++ -ansi -Wall -pedantic -O -c drawing.cpp  #
// #                                                                                   #
// #     letzte Aenderung:                07.07.2001                                   #
// #										       #
// #####################################################################################




// Include-Files

#include <iostream>									// Input / Output				
#include <fstream>									// File-Operationen
#include <iomanip>									// Manipulatoren fuer die Ein- und Ausgabe
#include <ctime>									// Zeitmessung
#include <cmath>									// mathematische Funktionen	
#include "drawing.h"									// Headerfile der Klasse drawing



// Funktionsrealisierungen

template<class T,class S> int Drawing<T,S>::getKnotengrad(int i)			// get-Methode
{
  // gibt den Knotengrad eines Knotens zurueck
  if(Knotengrad==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKnoten))							// Test auf moegliche Fehler	
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf den Knotengrad eines nicht existierenden Knoten zuzugreifen." << endl;
    return -1;
  };
  return Knotengrad[i];
};

template<class T,class S> int Drawing<T,S>::getAnzahlDerNachbarn(T i)			// get-Methode
{
  // gibt den Knotengrad eines Kreuzungspunktes zurueck
  if(AnzahlDerNachbarn==NULL)								// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKreuzungspunkte))						// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf den Knotengrad eines nicht existierenden Kreuzungspunktes zuzugreifen." << endl;
    return -1;
  };
  return AnzahlDerNachbarn[i];
};

template<class T,class S> int Drawing<T,S>::Schnittpunktfinden(S x1,S y1,S x2,S y2,S x3,S y3,S x4,S y4,S& s1,S& s2)	// interne Methode
{
  // Versucht zu 4 gegebenen Punkten den Schnitppunkt zu finden
  S abstand1,abstand2;
  if(abs(x1-x2)<FASTNULL)
  {
    if(abs(x4-x3)<FASTNULL)
      return -1;									// kein Schnittpunkt
    abstand2=(x1-x3)/(x4-x3);
    abstand1=((y1-y3)-(y4-y3)*abstand2)/(y1-y2);
  }
  else
  {
    if(abs(y1-y2)<FASTNULL)
    {
      if(abs(y4-y3)<FASTNULL)
        return -1;									// kein Schnittpunkt
      abstand2=(y1-y3)/(y4-y3);
      abstand1=((x1-x3)-(x4-x3)*abstand2)/(x1-x2);
    }
    else
    {
      if(abs((y4-y3)*(x1-x2)-(x4-x3)*(y1-y2))<10*FASTNULL)
        return -1;									// kein Schnittpunkt
      abstand2=((y1-y3)*(x1-x2)-(x1-x3)*(y1-y2))/((y4-y3)*(x1-x2)-(x4-x3)*(y1-y2));
      abstand1=((x1-x3)-(x4-x3)*abstand2)/(x1-x2);
    };
  };
  if((abstand2>1) || (abstand2<0) || (abstand1<0) || (abstand1>1))
    return -1;										// kein innerer Schnittpunkt
  s1=x1+abstand1*(x2-x1);
  s2=y1+abstand1*(y2-y1);
  // ist es wirklich ein neuer Schnittpunkt?
  // mu erst spaeter behandelt werden
  return 0;
};

template<class T,class S> T Drawing<T,S>::AnzahlIntersectionsRegularPolygon(int n)	// Berechnung
{
  // Berechnet die Anzahl der Kreuzungspunkte (intersection points) eines regulaeren Polygons nach
  // einer Formel aus
  // "The number of intersection points made by the diagonals of a regular polygon" [1]
  // Bemerkumg: Es wurde ein Druckfehler berichtigt
  long r;
  r=n*(n-1)/2*(n-2)/3*(n-3)/4;
  r+=delta(n,2)*(-5*n*n*n+45*n*n-70*n+24)/24-delta(n,4)*(3*n/2)+(-45*n*n+262*n)/6*delta(n,6);
  r+=42*n*delta(n,12)+60*n*delta(n,18)+35*n*delta(n,24)-38*n*delta(n,30)-82*n*delta(n,42);
  r+=-330*n*delta(n,60)-144*n*delta(n,84)-96*n*delta(n,90)-144*n*delta(n,120)-96*n*delta(n,210);
  return (T)r;
};

template<class T,class S> T Drawing<T,S>::AnzahlRegionsRegularPolygon(int n)		// Berechnung
{
  // Berechnet die Anzahl der Regionen (Flchen) eines regulaeren Polygons nach
  // einer Formel aus
  // "The number of intersection points made by the diagonals of a regular polygon" [1]
  long r;
  r=(n*n*n*n-6*n*n*n+23*n*n-42*n+24)/24;
  r+=delta(n,2)*(-5*n*n*n+42*n*n-40*n-48)/48-delta(n,4)*(3*n/4)+(-53*n*n+310*n)/12*delta(n,6);
  r+=49*n/2*delta(n,12)+32*n*delta(n,18)+19*n*delta(n,24)-36*n*delta(n,30)-50*n*delta(n,42);
  r+=-190*n*delta(n,60)-78*n*delta(n,84)-48*n*delta(n,90)-78*n*delta(n,120)-48*n*delta(n,210);
  return (T)r;
};

template<class T,class S> int Drawing<T,S>::delta(int n,int m)				// interne Berechnung
{
  // Funktion aus "The number of intersection points made by the diagonals of a regular polygon" [1]
  // gibt 1 zurueck, wenn n modulo m =0 und 1 sonst
  if((n%m)==0) return 1;
  return 0;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon1(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.BerechneNachbarArrays_o1();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer Speicherplatz optimierten Variante
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon2(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer nicht Speicherplatz optimierten Var	
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon1(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.BerechneNachbarArrays_o2();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke_o();							// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if(((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon2(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke();							// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if(((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon3(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte      ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_o1();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Anzahl der Polygone: " << rueck << endl;
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Polygone  ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon4(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer nicht Speicherplatz optimierten Var	
  time(&TimeGes);
  cout << "Anzahl der Polygone: " << rueck << endl;
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Polygone:   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon3(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte      ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_o2();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke_o();							// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if(((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke  ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon4(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke();							// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if(((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke:   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon5(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  d.BerechneNachbarArrays_o1();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer Speicherplatz optimierten Variante
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon6(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer nicht Speicherplatz optimierten Var	
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon5(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  d.BerechneNachbarArrays_o2();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke_o();							// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if(((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon6(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke();							// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if(((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon7(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte      ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Verschmelzen                       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_o1();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Anzahl der Polygone: " << rueck << endl;
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Polygone  ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlPolygoneRegularPolygon8(int n)		// Berechnung
{
  // Berechnet die Anzahl der Polygone eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Verschmelzen                       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlPolygone();							// Berechnet die Anzahl der Polygone mit einer nicht Speicherplatz optimierten Var	
  time(&TimeGes);
  cout << "Anzahl der Polygone: " << rueck << endl;
  int w=-1;
  for(int i=3;i<d.getAnzahlDerKnoten();i++)							// Testen des Ergebnisses
    if((d.getPolygone(i)%d.getAnzahlDerKnoten())!=0) w=i;
  if(w==-1)
    cout << "Test OK. Die Anzahl der m-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der " << w << "-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  if(rueck==d.AnzahlRegionsRegularPolygon(n))						// Testen des Ergebnisses
    cout << "Test OK. Die Gesamtanzahl der Polygone stimmt mit der Formel von Poonen-Rubinstein ueberein." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Gesamtanzahl der Polygone stimmt nicht mit der Formel von Poonen-Rubinstein ueberein." << endl;
    char taste;
    cin  >> taste;
  };
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Polygone:   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon7(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte      ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Verschmelzen                       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_o2();								// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke_o();							// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if(((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke  ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon8(int n)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken
  // nicht Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht noch keinen Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte();								// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Verschmelzen                       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays();								// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke();							// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  if( ((rueck%d.getAnzahlDerKnoten())==0) || (d.getAnzahlDerKnoten()<=3))		// Testen des Ergebnisses
    cout << "Test OK. Die Anzahl der 3-Ecke laesst sich durch die Anzahl der Knoten teilen." << endl;
  else
  {
    cout << "Fehler:" << endl;
    cout << "Die Anzahl der 3-Ecke ist nicht durch " << AnzahlDerKnoten << " teilbar." << endl;
    char taste;
    cin  >> taste;
  };
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke:   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

int SpeicherFehler(void)								// Standard-Fehlerausgabe
{
  // gibt eine Fehlermeldung aus, wenn benoetigter Speicherplatz nicht reserviert werden konnte
  cout << "Vom Programm benoetigter Speicherplatz konnte nicht alloziert werden." << endl;
  cout << "Bitte beenden Sie das Programm.";
  char taste;
  cin  >> taste;
  return -1;
};

template<class T,class S> int Drawing<T,S>::QS_x(T* F,T l,T r)				// Sortier-Methode
{
  // Sortiert eine Gerade nach X-Koordinaten
  // Quicksort nach Sedgewick
  T i,j;										// Schleifenvariablen
  S x;                                                                       		// zum Zwischenspeichern
  // Bestimme Pivotelement mit 3-Medianstrategie
  if(r-l>1)										// wenn ueberhaupt eine Auswahl vorhanden ist
  {
    if(IKnotenX(F[l])<IKnotenX(F[r]))
    {
      if(IKnotenX(F[l+T((r-l)/2)])>IKnotenX(F[r]))
        swap(F[r],F[l]);
      else
      {
        if(IKnotenX(F[l+T((r-l)/2)])>IKnotenX(F[l]))
          swap(F[l],F[l+T((r-l)/2)]);
      };
    }
    else
    {
      if(IKnotenX(F[l+T((r-l)/2)])<IKnotenX(F[l]))
      {
        if(IKnotenX(F[l+T((r-l)/2)])<IKnotenX(F[r]))
          swap(F[r],F[l]);
        else
          swap(F[l],F[l+T((r-l)/2)]);
      };
    };
  };
  x=IKnotenX(F[l]);									// Merke diesen Wert
  i=l;
  j=r;
  while(true)
  {
    while((i<=r) && (IKnotenX(F[i])<=x))						// notwendig, da kein Feldelement unednlich benutzt wird
      i++;
    while(IKnotenX(F[j])>x)
      j--;
    if(j<=i)
      break;
    else
      swap(F[i],F[j]);                 							// vertauschen
  };
  swap(F[l],F[j]);									// vertauschen
  if(j-1-l>0) QS_x(F,l,j-1);								// wenn ueberhaupt noch was zu tun ist
  if(r-j-1>0) QS_x(F,j+1,r);            						// wenn ueberhaupt noch was zu tun ist
  return 0;
};

template<class T,class S> int Drawing<T,S>::QS_y(T* F,T l,T r)				// Sortier-Methode
{
  // Sortiert eine Gerade nach Y-Koordinaten
  // Quicksort nach Sedgewick
  T i,j;										// Schleifenvariablen
  S x;                                                                        		// zum Zwischenspeichern
  // Bestimme Pivotelement mit 3-Medianstrategie
  if(r-l>1)										// wenn es ueberhaupt eine Auswahl gibt
  {
    if(IKnotenY(F[l])<IKnotenY(F[r]))
    {
      if(IKnotenY(F[l+T((r-l)/2)])>IKnotenY(F[r]))
        swap(F[r],F[l]);
      else
      {
        if(IKnotenY(F[l+T((r-l)/2)])>IKnotenY(F[l]))
          swap(F[l],F[l+T((r-l)/2)]);
      };
    }
    else
    {
      if(IKnotenY(F[l+T((r-l)/2)])<IKnotenY(F[l]))
      {
        if(IKnotenY(F[l+T((r-l)/2)])<IKnotenY(F[r]))
          swap(F[r],F[l]);
        else
          swap(F[l],F[l+T((r-l)/2)]);
      };
    };
  };
  x=IKnotenY(F[l]);									// Merke diesn Wert
  i=l;
  j=r;
  while(true)
  {
    while((i<=r) && (IKnotenY(F[i])<=x))						// notwendig, da kein Feldelement unednlich benutzt wird
      i++;
    while(IKnotenY(F[j])>x)
      j--;
    if(j<=i)
      break;
    else
      swap(F[i],F[j]);                 							// vertauschen
  };
  swap(F[l],F[j]);									// vertauschen
  if(j-1-l>0) QS_y(F,l,j-1);								// wenn ueberhaupt noch was zu tun ist
  if(r-j-1>0) QS_y(F,j+1,r);            						// wenn ueberhaupt noch was zu tun ist
  return 0;
};

template<class T,class S> int Drawing<T,S>::MS_y(T* F,T l,T r)				// Sortier-Methode
{
  // Sortiert eine Gerade nach Y-Koordinaten
  // rekursives Mergesort
  T i,j,k;             									// Schleifenvariablen
  if(r-l>1)										// wenn ueberhaupt noch etwas zu tun ist
    MS_y(F,l,l+T((r-l)/2));
  if(r-l>1)										// wenn ueberhaupt noch etwas zu tun ist
    MS_y(F,l+T((r-l)/2)+1,r);
  T* H;
  if((H=new T[r-l+1])==0) SpeicherFehler();
  i=l;
  j=T((r-l)/2)+1+l;
  k=0;
  // mische die zwei sortierten Felder zusammen
  while(k<=r-l)
  {
    while((i<=l+T((r-l)/2)) && (IKnotenY(F[i])<=IKnotenY(F[j])))
    {
      H[k]=F[i];
      k++;
      i++;
    };
    if(i>l+T((r-l)/2))
    {
      while(j<=r)
      {
        H[k]=F[j];
        k++;
        j++;
      };
    };
    while((j<=r) && (IKnotenY(F[j])<IKnotenY(F[i])))
    {
      H[k]=F[j];
      k++;
      j++;
    };
    if(j>r)
    {
      while(i<=l+T((r-l)/2))
      {
        H[k]=F[i];
        k++;
        i++;
      };
    };
  };
  for(i=0;i<=r-l;i++)
    F[l+i]=H[i];
  if(H!=NULL) delete[] H;
  H=NULL;
  return 0;
};

template<class T,class S> int Drawing<T,S>::MS_x(T* F,T l,T r)				// Sortier-Methode
{
  // Sortiert eine Gerade nach X-Koordinaten
  // rekursives Mergesort
  T i,j,k;										// Schleifenvariablen
  if(r-l>1)										// wenn ueberhaupt etwas zu tun ist
    MS_x(F,l,l+T((r-l)/2));
  if(r-l>1)										// wenn ueberhaupt etwas zu tun ist	
    MS_x(F,l+T((r-l)/2)+1,r);
  T* H;
  if((H=new T[r-l+1])==0) SpeicherFehler();
  i=l;
  j=T((r-l)/2)+1+l;
  k=0;
  // mische die zwei sortierten Felder zusammen
  while(k<=r-l)
  {
    while((i<=l+T((r-l)/2)) && (IKnotenX(F[i])<=IKnotenX(F[j])))
    {
      H[k]=F[i];
      k++;
      i++;
    };
    if(i>l+T((r-l)/2))
    {
      while(j<=r)
      {
        H[k]=F[j];
        k++;
        j++;
      };
    };
    while((j<=r) && (IKnotenX(F[j])<IKnotenX(F[i])))
    {
      H[k]=F[j];
      k++;
      j++;
    };
    if(j>r)
    {
      while(i<=l+T((r-l)/2))
      {
        H[k]=F[i];
        k++;
        i++;
      };
    };
  };
  for(i=0;i<=r-l;i++)
    F[l+i]=H[i];
  if(H!=NULL) delete[] H;
  H=NULL;
  return 0;
};

template<class T,class S> int Drawing<T,S>::HS_x(T* F,T laenge)                		// Sortier-Methode
{
  // Sortieren einer Gerade nach der X-Koordinate mit Heapsort	
  T l,j,k;                                                    				// Schleifenvariablen
  T x;				
  T s=T(laenge/2)-1;

  // Herstellen eines Haufens
  for(l=s;l>=0;l--)
  {
    j=l;
    x=F[j];
    while(2*j+1<laenge)
    {
      k=2*j+1;
      if((k+1<laenge) & (IKnotenX(F[k])<IKnotenX(F[k+1])))
        k++;
      if(IKnotenX(x)<IKnotenX(F[k]))
      {
        x=F[j];
        swap(F[j],F[k]);
      }
      else
        break;
      j=k;
    };
  };

  // Sortieren des Haufens
  for(l=laenge-1;l>0;l--)
  {
    x=F[0];
    F[0]=F[l];
    F[l]=x;
    j=0;
    x=F[j];
    while(2*j+1<l)
    {
      k=2*j+1;
      if((k+1<l) &(IKnotenX(F[k])<IKnotenX(F[k+1])))
        k++;
      if(IKnotenX(x)<IKnotenX(F[k]))
      {
        x=F[j];
        swap(F[j],F[k]);
      }
      else
        break;
      j=k;
    };
  };
  return 0;
};

template<class T,class S> int Drawing<T,S>::HS_y(T* F,T laenge)				// Sortier-Methode
{
  // Sortieren einer Geraden nach der Y-Koordinate mit Heapsort
  T l,j,k;                                          					// Schleifenvariablen
  T x;				
  T s=T(laenge/2)-1;

  // Herstellen eines Haufens
  for(l=s;l>=0;l--)
  {
    j=l;
    x=F[j];
    while(2*j+1<laenge)
    {
      k=2*j+1;
      if((k+1<laenge) & (IKnotenY(F[k])<IKnotenY(F[k+1])))
        k++;
      if(IKnotenY(x)<IKnotenY(F[k]))
      {
        x=F[j];
        swap(F[j],F[k]);
      }
      else
        break;
      j=k;
    };
  };

  // Sortieren des Haufens
  for(l=laenge-1;l>0;l--)
  {
    x=F[0];
    F[0]=F[l];
    F[l]=x;
   j=0;
    x=F[j];
    while(2*j+1<l)
    {
      k=2*j+1;
      if((k+1<l) &(IKnotenY(F[k])<IKnotenY(F[k+1])))
        k++;
      if(IKnotenY(x)<IKnotenY(F[k]))
      {
        x=F[j];
        swap(F[j],F[k]);
      }
      else
        break;
      j=k;
    };
  };
 return 0;
};

template<class T> inline T min(T a,T b)							// interne Methode
{
  // gibt das kleinere von zwei Elementen zurueck
  if(a<b) return a;
  return b;
};

template<class T> inline int swap(T& a,T& b)						// interne Methode
{
  // Vertausche a und b
  T x;  										// allgemeiner Datentyp
  x=a;
  a=b;
  b=x;
  return 0;
};

template<class T,class S> int Drawing<T,S>::setMatrix(int i,int j,int w)		// set-Methode
{
  // setzt den Wert w an die Position (i,j) der Verbindungs-Matrix
  int h;
  if((i<0) || (j<0) || (i>=AnzahlDerKnoten) || (j>=AnzahlDerKnoten))			// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht einen nicht definiertes Element der Matrix eines Drawing-Objekts zu belegen." << endl;
    return -1;
  };
  if((w!=0) && (w!=1))									// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht die Matrix eines Drawing-Objekts mit einem Wert verschieden von 0,1 zu belegen." << endl;
    return -1;
  };
  if(i==j)										// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht einen Loop in die Matrix eines Drawing-Objekts einzufuegen." << endl;
    return -1;
  };
  if(Matrix==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf die Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
    return -1;
  };
  for(h=0;h<AnzahlDerKnoten;h++)							// Test auf moegliche Fehler
    if(Matrix[h]==NULL)
    {
      cout << "Fehler:" << endl;
      cout << "Es wurde versucht auf die eine Spalte der Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
      return -1;
    };
  Matrix[i][j]=w;
  Matrix[j][i]=w;									// die Matrix beschreibt einen ungerichteten Graphen, daher ist sie symmetrisch
  return 0;
};

template<class T,class S> int Drawing<T,S>::setMatrix(int i,int* w)			// set-Methode
{
  // setzt den Vektor w an die i-te Spalte der Verbindungs-Matrix
  int h;
  if((i<0)  || (i>=AnzahlDerKnoten) )							// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht einen nicht definiertes Element der Matrix eines Drawing-Objekts zu belegen." << endl;
    return -1;
  };
  if(Matrix==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf die Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
    return -1;
  };
  for(h=0;h<AnzahlDerKnoten;h++)							// Test auf moegliche Fehler
    if(Matrix[h]==NULL)
    {
      cout << "Fehler:" << endl;
      cout << "Es wurde versucht auf die eine Spalte der Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
      return -1;
    };
  for(int j=0;j<AnzahlDerKnoten;j++)
  {
    if((w[j]!=0) && (w[j]!=1))								// Test auf moegliche Fehler
    {
      cerr << "Fehler:" << endl;
      cerr << "Es wurde versucht die Matrix eines Drawing-Objekts mit einem Wert verschieden von 0,1 zu belegen." << endl;
      return -1;
    };
    if(i!=j)
    {
      Matrix[i][j]=w[j];
      Matrix[j][i]=w[j];
    }
    else
    {
      Matrix[j][j]=0;									// Loops sind nicht erlaubt
    };
  };
  return 0;
};

template<class T,class S> int Drawing<T,S>::setMatrix(int** w)				// set-Methode	
{
  // belegt die Werte der Verbindungs-Matrix mit den Werten aus w
  int h;
  if(Matrix==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf die Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
    return -1;
  };
  for(h=0;h<AnzahlDerKnoten;h++)							// Test auf moegliche Fehler
    if(Matrix[h]==NULL)
    {
      cout << "Fehler:" << endl;
      cout << "Es wurde versucht auf die eine Spalte der Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
      return -1;
    };
  for(int i=0;i<AnzahlDerKnoten;i++)
    for(int j=0;j<AnzahlDerKnoten;j++)
    {
      if((w[i][j]!=0) && (w[i][j]!=1))       						// Test auf moegliche Fehler
      {
        cerr << "Fehler:" << endl;
        cerr << "Es wurde versucht die Matrix eines Drawing-Objekts mit einem Wert verschieden von 0,1 zu belegen." << endl;
        return -1;
      };
      if(w[i][j]!=w[j][i])								// Test auf moegliche Fehler
      {
        cerr << "Fehler:" << endl;
        cerr << "Es wurde versucht die Matrix eines Drawing-Objekts mit nicht symmetrischen Werten zu belegen." << endl;
        return -1;
      };
      if(i!=j)
        Matrix[i][j]=w[i][j];
     else
       Matrix[i][i]=0;									// Loops sind nicht erlaubt
    };
  return 0;
};

template<class T,class S> int Drawing<T,S>::CreateCompleteGraph(void)			// Erzeuge-Methode
{
  // erzeugt die Verbindungs-Matrix eines vollstaendigen Graphen
  int i;
  if(Matrix==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf die Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
    return -1;
  };
  for(i=0;i<AnzahlDerKnoten;i++)							// Test auf moegliche Fehler
    if(Matrix[i]==NULL)
    {
      cout << "Fehler:" << endl;
      cout << "Es wurde versucht auf die eine Spalte der Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
      return -1;
    };
  for(i=0;i<AnzahlDerKnoten;i++)
    for(int j=0;j<AnzahlDerKnoten;j++)
      Matrix[i][j]=1;
  for(i=0;i<AnzahlDerKnoten;i++)
    Matrix[i][i]=0;
  return 0;
};

template<class T,class S> Drawing<T,S>::Drawing(void)					// Standard-Konstruktor
{
  AnzahlDerKnoten=1;
  init(AnzahlDerKnoten);								// initialisere
};

template<class T,class S> Drawing<T,S>::Drawing(int n)					// Konstruktor
{
  if(n<=0)										// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr <<  "Es wurde versucht ein Drawing-Objekt mit einer negativen Anzahl von Knoten zu erzeugen." << endl;
    n=1;
  }
  else
    init(n);										// initialisiere
};

template<class T,class S> T Drawing<T,S>::getAnzahlDerKreuzungspunkte(void)		// get-Methode
{
  // gibt die Anzahl der Kreuzungspunkte zurueck
  return AnzahlDerKreuzungspunkte;
};

template<class T,class S> T Drawing<T,S>::getAnzahlDerKanten(void)			// get-Methode
{
  // gibt die Anzahl der Kanten zurueck
  return AnzahlDerKanten;
};

template<class T,class S> int Drawing<T,S>::getMatrix(int i,int j)			// get-Methode
{
  // gibt den Wert der Verbindungs-Matrix an der Position (i,j) zurueck
  int h;
  if(Matrix==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf die Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
    return -1;
  };
  for(h=0;h<AnzahlDerKnoten;h++)							// Test auf moegliche Fehler
    if(Matrix[h]==NULL)
    {
      cout << "Fehler:" << endl;
      cout << "Es wurde versucht auf die eine Spalte der Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
      return -1;
    };
  if((i<0) || (j<0) || (i>=AnzahlDerKnoten) || (j>=AnzahlDerKnoten))   			// Test auf moegliche Fehler	
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht auf ein nicht definiertes Element der Matrix eines Drawing-Objekts zuzugreifen." << endl;
    return -1;
  };
  return Matrix[i][j];
};

template<class T,class S> int Drawing<T,S>::getAnzahlDerKnoten(void)			// get-Methode
{
  // gibt die Anzahl der Knoten zurueck
  return AnzahlDerKnoten;
};

template<class T,class S> S Drawing<T,S>::getKnotenX(int i)				// get-Methode
{
  // gibt die X-Koordinate eines Knotens zurueck
  if(KnotenX==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKnoten))             					// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht auf die X-Koordinate eines nicht definierten Knotens in einem Drawing-Objekt zuzugreifen." << endl;
    return -1;
  };
  return KnotenX[i];
};

template<class T,class S> S Drawing<T,S>::getKnotenY(int i)				// get-Methode
{
  // gibt die Y-Koordinate eines Knotens zurueck
  if(KnotenY==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKnoten))             					// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht auf die Y-Koordinate eines nicht definierten Knotens in einem Drawing-Objekt zuzugreifen." << endl;
    return -1;
  };
  return KnotenY[i];
};

template<class T,class S> S Drawing<T,S>::getKreuzungspunktX(T i)			// get-Methode
{
  // gibt die X-Koordinate eines Kreuzungspunktes zurueck
  if(KreuzungspunktX==NULL)								// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKreuzungspunkte))             				// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht auf die X-Koordinate eines nicht definierten Kreuzungspunktes in einem Drawing-Objekt zuzugreifen." << endl;
    return -1;
  };
  return KreuzungspunktX[i];
};

template<class T,class S> S Drawing<T,S>::getKreuzungspunktY(T i)			// get-Methode
{
  // gibt die Y-Koordinate eines Kreuzungspunktes zurueck
  if(KreuzungspunktY==NULL)								// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKreuzungspunkte))             				// Test auf moegliche Fehler
  {
    cerr << "Fehler:" << endl;
    cerr << "Es wurde versucht auf die Y-Koordinate eines nicht definierten Kreuzungspunktes in einem Drawing-Objekt zuzugreifen." << endl;
    return -1;
  };
  return KreuzungspunktY[i];
};

template<class T,class S> T Drawing<T,S>::Nachbar(T i,T e)				// interne Routine
{
  // gibt den e-ten Nachbarn des Knoten oder Kreuzungspunktes i zurueck
  if((Nachbar1==NULL) || (Nachbar2==NULL))						// Test auf moegliche Fehler	
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return 0;
  };
  if((i<0) || (i>=AnzahlDerKnoten+AnzahlDerKreuzungspunkte))				// Test auf moegliche Fehler
  {
    cerr << "Fehler in Klasse Drawing:" << endl;
    cerr << "Es wurde versucht auf einen Nachbarn eines nicht existierenden Knotens zuzugreifen." << endl;
    return -1;
  };
  if((e<0) || (e>=IKnotengrad(i)))							// Test auf moegliche Fehler
  {
    cerr << "Fehler in Klasse Drawing:" << endl;
    cerr << "Es wurde versucht auf einen nicht existierenden Nachbarn eines Knotens zuzugreifen." << endl;
    return -1;
  };
  return Nachbar1[Nachbar2[i]+e];
};

template<class T,class S> S Drawing<T,S>::IKnotenX(T i)					// interne Routine
{
  // gibt die X-Koordinate eines Knotens oder Kreuzungspunktes zurueck
  if((KnotenX==NULL) || (KreuzungspunktX==NULL))					// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKnoten+AnzahlDerKreuzungspunkte))				// Test auf moegliche Fehler	
  {
    cerr << "Fehler in der Klasse Drawing:" << endl;
    cerr << "Es wurde versucht auf die X-Koordinate eines nicht exiistierenden Punktes zuzufgreifen." << endl;
    return -1;
  };
  if(i<AnzahlDerKnoten) return KnotenX[i];						// i ist Knoten
  return KreuzungspunktX[i-AnzahlDerKnoten];                   			 	// i ist Kreuzungspunkt
};

template<class T,class S> S Drawing<T,S>::IKnotenY(T i)					// interne Routine
{
  // gibt die Y-Koordinate eines Knotens oder Kreuzungspunktes zurueck
  if((KnotenY==NULL) || (KreuzungspunktY==NULL))					// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<0) || (i>=AnzahlDerKnoten+AnzahlDerKreuzungspunkte))          			// Test auf moegliche Fehler
  {
    cerr << "Fehler in der Klasse Drawing:" << endl;
    cerr << "Es wurde versucht auf die Y-Koordinate eines nicht exiistierenden Punktes zuzufgreifen." << endl;
    return -1;
  };
  if(i<AnzahlDerKnoten) return KnotenY[i];						// i ist Knoten
  return KreuzungspunktY[i-AnzahlDerKnoten];                    			// i ist Kreuzungspunkt
};

template<class T,class S> int Drawing<T,S>::setIKnotenX(T i,S w)			// interne Methode
{
  // setzt die X-Koordinate eines Knotens oder Kreuzungspunktes
  if((i<AnzahlDerKnoten) && (i>=0)) KnotenX[i]=w;
  if((i>=AnzahlDerKnoten) && (i<AnzahlDerKnoten+AnzahlDerKreuzungspunkte)) KreuzungspunktX[i-AnzahlDerKnoten]=w;
  return 0;
};

template<class T,class S> int Drawing<T,S>::setIKnotenY(T i,S w)			// interne Methode
{
  // setzt die Y-Koordinate eines Knotens oder Kreuzungspunktes
  if((i<AnzahlDerKnoten) && (i>=0)) KnotenY[i]=w;
  if((i>=AnzahlDerKnoten) && (i<AnzahlDerKnoten+AnzahlDerKreuzungspunkte)) KreuzungspunktY[i-AnzahlDerKnoten]=w;
  return 0;
};


template<class T,class S> S Drawing<T,S>::IKreuzungspunktX(T i)				// interne Routine
{
  // gibt die X-Koordinate eines Knotens oder Kreuzungspunktes zurueck
  return IKnotenX(i);
};

template<class T,class S> S Drawing<T,S>::IKreuzungspunktY(T i)  			// interne Routine
{
  // gibt die Y-Koordinate eines Knotens oder Kreuzungspunktes zurueck
  return IKnotenY(i);
};

template<class T,class S> int Drawing<T,S>::IKnotengrad(T i)				// interne Routine
{
  // gibt den Knotengrad eines Knotens oder Kreuzungspunktes zurueck
  if((Knotengrad==NULL) || (AnzahlDerNachbarn==NULL))					// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return 0;
  };
  if((i<0) || (i>=AnzahlDerKnoten+AnzahlDerKreuzungspunkte))				// Test auf moegliche Fehler
  {
    cerr << "Fehler in der Klasse Drawing:" << endl;
    cerr << "Es wurde versucht auf den Knotengrad eines nicht exiistierenden Punktes zuzufgreifen." << endl;
    cerr << i << endl;
    return -1;
  };
  if(i<AnzahlDerKnoten) return Knotengrad[i];   					// i ist Knoten	
  return AnzahlDerNachbarn[i-AnzahlDerKnoten];          				// i ist Kreuzungspunkt
};

template<class T,class S> int Drawing<T,S>::IAnzahlDerNachbarn(T i)			// interne Routine
{
  // gibt den Knotengrad eines Knotens oder Kreuzungspunktes zurueck
  return IKnotengrad(i);
};

template<class T,class S> int Drawing<T,S>::sortiereGeradenQS(void)			// interne Methode
{
  // sortiert die Geraden mit Quicksort
  if((Gerade1==NULL) || (Gerade2==NULL))						// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  T i,j;     										// Schleifenvariablen				
  T* speicher7;										// Zischwenspeicher fuers sortieren
  speicher7=new T[AnzahlDerKnoten*(AnzahlDerKnoten-1)/2];				// dies ist die allgemeine Maximalzahl von Kreuzungspunkten auf einer Geraden
  for(i=0;i<AnzahlDerKanten;i++)							// durchlaufe alle Kanten
  {
    for(j=0;j<Gerade1[Gerade2[i]];j++)
      speicher7[j]=Gerade1[Gerade2[i]+j+1];
    if(Gerade1[Gerade2[i]]>1)                                                      	// mu ueberhaupt sortiert werden?
      if(abs(IKnotenX(Gerade1[Gerade2[i]+1])-IKnotenX(Gerade1[Gerade2[i]+2]))>FASTNULL)	
      {
        QS_x(speicher7,0,Gerade1[Gerade2[i]]-1);                          		// Sortierung nach der X-Koordinate
        if(IKnotenX(Strecke1[i])>IKnotenX(Strecke2[i]))					// Orientierung der Anfangs- und Endknoten
          swap(Strecke1[i],Strecke2[i]);
      }
      else		
      {
        QS_y(speicher7,0,Gerade1[Gerade2[i]]-1);					// Sortierung nach der Y-Koordinate
        if(IKnotenY(Strecke1[i])>IKnotenY(Strecke2[i]))					// Orientierung der Anfangs- und Endknoten
          swap(Strecke1[i],Strecke2[i]);
      };
    for(j=0;j<Gerade1[Gerade2[i]];j++)
      Gerade1[Gerade2[i]+j+1]=speicher7[j];
  };
  if(speicher7!=NULL) delete[] speicher7;						// Speicherfreigabe fuer den Zwischenspeicher
  speicher7=NULL;
  return 0;
};

template<class T,class S> int Drawing<T,S>::sortiereGeradenMS(void)			// interne Methode
{
  // sortiert die Geraden mit Mergesort
  if((Gerade1==NULL) || (Gerade2==NULL))						// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  T i,j;     										// Schleifenvariablen				
  T* speicher7;										// Zischwenspeicher fuers sortieren
  speicher7=new T[AnzahlDerKnoten*(AnzahlDerKnoten-1)/2];				// dies ist die allgemeine Maximalzahl von Kreuzungspunkten auf einer Geraden
  for(i=0;i<AnzahlDerKanten;i++)							// durchlaufe alle Kanten
  {
    for(j=0;j<Gerade1[Gerade2[i]];j++)
      speicher7[j]=Gerade1[Gerade2[i]+j+1];
    if(Gerade1[Gerade2[i]]>1)                                                      	// mu ueberhaupt sortiert werden?
      if(abs(IKnotenX(Gerade1[Gerade2[i]+1])-IKnotenX(Gerade1[Gerade2[i]+2]))>FASTNULL)	
      {
        MS_x(speicher7,0,Gerade1[Gerade2[i]]-1);                          		// Sortierung nach der X-Koordinate
        if(IKnotenX(Strecke1[i])>IKnotenX(Strecke2[i]))					// Orientierung der Anfangs- und Endknoten
          swap(Strecke1[i],Strecke2[i]);
      }
      else		
      {
        MS_y(speicher7,0,Gerade1[Gerade2[i]]-1);					// Sortierung nach der Y-Koordinate
        if(IKnotenY(Strecke1[i])>IKnotenY(Strecke2[i]))					// Orientierung der Anfangs- und Endknoten
          swap(Strecke1[i],Strecke2[i]);
      };
    for(j=0;j<Gerade1[Gerade2[i]];j++)
      Gerade1[Gerade2[i]+j+1]=speicher7[j];
  };
  if(speicher7!=NULL) delete[] speicher7;						// Speicherfreigabe fuer den Zwischenspeicher
  speicher7=NULL;
  return 0;
};

template<class T,class S> int Drawing<T,S>::sortiereGeradenHS(void)			// interne Methode
{
  // sortiert die Geraden mit Heapsort
  if((Gerade1==NULL) || (Gerade2==NULL))						// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  T i,j;     										// Schleifenvariablen				
  T* speicher7;										// Zischwenspeicher fuers sortieren
  speicher7=new T[AnzahlDerKnoten*(AnzahlDerKnoten-1)/2];				// dies ist die allgemeine Maximalzahl von Kreuzungspunkten auf einer Geraden
  for(i=0;i<AnzahlDerKanten;i++)							// durchlaufe alle Kanten
  {
    for(j=0;j<Gerade1[Gerade2[i]];j++)
      speicher7[j]=Gerade1[Gerade2[i]+j+1];
    if(Gerade1[Gerade2[i]]>1)                                                      	// mu ueberhaupt sortiert werden?
      if(abs(IKnotenX(Gerade1[Gerade2[i]+1])-IKnotenX(Gerade1[Gerade2[i]+2]))>FASTNULL)	
      {
        HS_x(speicher7,Gerade1[Gerade2[i]]);                          			// Sortierung nach der X-Koordinate
        if(IKnotenX(Strecke1[i])>IKnotenX(Strecke2[i]))					// Orientierung der Anfangs- und Endknoten
          swap(Strecke1[i],Strecke2[i]);
      }
      else		
      {
        HS_y(speicher7,Gerade1[Gerade2[i]]);						// Sortierung nach der Y-Koordinate
        if(IKnotenY(Strecke1[i])>IKnotenY(Strecke2[i]))					// Orientierung der Anfangs- und Endknoten
          swap(Strecke1[i],Strecke2[i]);
      };
    for(j=0;j<Gerade1[Gerade2[i]];j++)
      Gerade1[Gerade2[i]+j+1]=speicher7[j];
  };
  if(speicher7!=NULL) delete[] speicher7;						// Speicherfreigabe fuer den Zwischenspeicher
  speicher7=NULL;
  return 0;
};

template<class T,class S> T Drawing<T,S>::BerechneZaehler(T ii)				// interne Methode
{
  // Berechnet die Anzahl der Schnittpunkte auf dieser Kante
  if( (Strecke1==NULL) || (Strecke2==NULL) || (KnotenX==NULL) || (KnotenY==0) )		// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return 0;
  };
  T links,rechts;
  S wert;
  T i;
  links=rechts=0;
  for(i=0;i<AnzahlDerKnoten;i++)							// durchlaufe alle Knoten
  {
    if((i!=Strecke1[ii]) && (i!=Strecke2[ii]))                     			// auer den zwei Endknoten
    {
      if(abs(KnotenX[Strecke1[ii]]-KnotenX[Strecke2[ii]])<FASTNULL)			// ist Kante parallel zur Y-Achse?
      {
        if(KnotenX[i]<KnotenX[Strecke1[i]])
          links++;									// Knoten liegt links von der Kante
        else
          rechts++;									// Knoten liegt rechts von der Kante
      }
      else
      {
        wert=KnotenX[i]-KnotenX[Strecke1[ii]];						// Berechne Vergleichs- Y-Koordinate
        wert*=KnotenY[Strecke2[ii]]-KnotenY[Strecke1[ii]];
        wert/=KnotenX[Strecke2[ii]]-KnotenX[Strecke1[ii]];
        wert+=KnotenY[Strecke1[ii]];
        if(KnotenY[i]<wert)
          links++;									// Knoten liegt links von der Kante
        else
          rechts++;									// Knoten liegt rechts von der Kante
      };
    };
  };
  if(rechts+links!=AnzahlDerKnoten-2)							// Test auf moegliche Fehler
  {
    cerr << "Fehler on Klasse Drawing:" << endl;
    cerr << "BerechneZaehler hat ein unsinniges Ergebnis berechnet." << endl;
    char taste;
    cin  >> taste;
    return -1;
  };
  return rechts*links;									
};

template<class T,class S>int Drawing<T,S>::CreateRegularNGon(void)			// Erzeuge-Methode
{
  // Erzeugt ein regulaeres n-Eck
  CreateCompleteGraph();
  CreateRegularPoints();
  return 0;
};
template<class T,class S>int Drawing<T,S>::CreateRegularPoints(void)			// Erzeuge-Methode
{
  // Erzeugt die Koordinaten eines regelmaessigen n-Ecks
  if((KnotenX==NULL) || (KnotenY==NULL))
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein schon geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  for(int i=0;i<AnzahlDerKnoten;i++)
  {
    KnotenX[i]=sin(2*PI/AnzahlDerKnoten*i);
    KnotenY[i]=cos(2*PI/AnzahlDerKnoten*i);
  };
  return 0;
};

template<class T,class S> int Drawing<T,S>::BerechneKreuzungspunkte(void)		// Berechnung	
{
  // Berechnet die Kreuzungspunkte
  int h;
  if(Matrix==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf die Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
    return -1;
  };
  for(h=0;h<AnzahlDerKnoten;h++)							// Test auf moegliche Fehler
    if(Matrix[h]==NULL)
    {
      cout << "Fehler:" << endl;
      cout << "Es wurde versucht auf die eine Spalte der Verbindungsmatrix zuzugreifen, wobei diese schon geloescht wurden ist." << endl;
      return -1;
    };
  T i,j;										// Schleifenvariablen
  T max;           									// Hilfsvariable
  max=AnzahlDerKnoten*(AnzahlDerKnoten-1)/2;
  max*=AnzahlDerKnoten-2;
  max/=3;
  max*=AnzahlDerKnoten-3;
  max/=4;										// Berechne maximale Anzahl der Kreuzungspunkte
  if(KreuzungspunktX!=NULL) delete[] KreuzungspunktX;					// sicher ist sicher
  KreuzungspunktX=NULL;
  if(KreuzungspunktY!=NULL) delete[] KreuzungspunktY;                      		// sicher ist sicher
  KreuzungspunktY=NULL;
  if((KreuzungspunktX=new S[max])==0) SpeicherFehler();           			// reserviere Speicherplatz fuer die X-Koordinaten der Kreuzungspunkte
  if((KreuzungspunktY=new S[max])==0) SpeicherFehler();					// reserviere Speicherplatz fuer die Y-Koordinaten der Kreuzungspunkte
  if(Strecke1!=NULL) delete[] Strecke1;							// sicher ist sicher
  Strecke1=NULL;
  if(Strecke2!=NULL) delete[] Strecke2;                                                 // sicher ist sicher
  Strecke2=NULL;
  if((Strecke1=new T[AnzahlDerKnoten*(AnzahlDerKnoten-1)/2])==0) SpeicherFehler();  	// reserviere Speicherplatz fuer die Anfangsknoten der Geraden
  if((Strecke2=new T[AnzahlDerKnoten*(AnzahlDerKnoten-1)/2])==0) SpeicherFehler();	// reserviere Speicherplatz fuer die Endknoten der Geraden	
  AnzahlDerKanten=0;
  for(i=0;i<AnzahlDerKnoten-1;i++)
    for(j=i+1;j<AnzahlDerKnoten;j++)							// durchlaufe alle Paare von Knoten
      if(Matrix[i][j]==1)								// wenn die zwei Knoten durch eine Kante verbunden sind
      {
        Strecke1[AnzahlDerKanten]=i;							// Anfangsknoten
        Strecke2[AnzahlDerKanten]=j;                                                    // Endknoten
        GeradenNr[i][j]=AnzahlDerKanten;
        GeradenNr[j][i]=AnzahlDerKanten;                                                // eine Kante mehr
        AnzahlDerKanten++;
      };

  if(AnzahlDerKanten!=max)								// wenn die Anzahl der Kanten nicht ihr Maximum annimmt, kann man etwas Speicherplatz sparen
  {
    // spare etwas Speicher
    T* speicherP;									// Hilfsarray	
    if((speicherP=new T[AnzahlDerKanten])==0) SpeicherFehler();
    for(i=0;i<AnzahlDerKanten;i++)
      speicherP[i]=Strecke1[i];								// merke Strecke1
    if(Strecke1!=NULL) delete[] Strecke1;                            			// sicher ist sicher
    Strecke1=NULL;
    if((Strecke1=new T[AnzahlDerKanten])==0) SpeicherFehler();				// Speicherplatz reservieren
    for(i=0;i<AnzahlDerKanten;i++)
    {
      Strecke1[i]=speicherP[i];								// Strecke1 wieder belegen
      speicherP[i]=Strecke2[i];                                                         // merke Strecke2
    };
    if(Strecke2!=NULL) delete[] Strecke2;              					// sicher ist sicher
    Strecke2=NULL;
    if((Strecke2=new T[AnzahlDerKanten])==0) SpeicherFehler();    			// Speicherplatz reservieren
    for(i=0;i<AnzahlDerKanten;i++)
      Strecke2[i]=speicherP[i];     							// Strecke2 wieder belegen
    if(speicherP!=NULL) delete[] speicherP;                                             // sicher ist sicher
    speicherP=NULL;
  };

  if(Gerade2!=NULL) delete[] Gerade2;							// sicher ist sicher	
  Gerade2=NULL;
  if(Gerade1!=NULL) delete[] Gerade1;                                                   // sicher ist sicher
  Gerade1=NULL;
  if((Gerade2=new T[AnzahlDerKanten])==0) SpeicherFehler();       			// Speicherplatz reservieren
  if((Gerade1=new T[2*max+AnzahlDerKnoten*AnzahlDerKnoten*AnzahlDerKnoten])==0) SpeicherFehler(); 	// Speicherplatz reservieren
  if(AnzahlDerNachbarn!=NULL) delete[] AnzahlDerNachbarn;				// sicher ist sicher
  AnzahlDerNachbarn=NULL;
  if((AnzahlDerNachbarn=new int[max+AnzahlDerKnoten*(AnzahlDerKnoten+1)/2])==0) SpeicherFehler();	// Speicherplatz reservieren
  j=0;
  for(i=0;i<AnzahlDerKanten;i++)
  {
    Gerade2[i]=j;
    Gerade1[j]=0;
    j+=BerechneZaehler(i)+1;								// Berechne die Anzahl der Schnittpunkte auf dieser Gerade
  };
  j--;
  for(i=0;i<j;i++)
    Gerade1[i]=0;

  // Schnittpunkte finden
  S x3,y3;
  T anzahl;
  AnzahlDerKreuzungspunkte=0;
  for(i=0;i<AnzahlDerKanten-1;i++)
  {
    for(j=i+1;j<AnzahlDerKanten;j++)
    {
      if((Strecke1[i]!=Strecke1[j]) && (Strecke2[i]!=Strecke1[j]) && (Strecke2[i]!=Strecke2[j]))
      {
        if(Schnittpunktfinden(KnotenX[Strecke1[i]],KnotenY[Strecke1[i]],KnotenX[Strecke2[i]],KnotenY[Strecke2[i]],KnotenX[Strecke1[j]],KnotenY[Strecke1[j]],KnotenX[Strecke2[j]],KnotenY[Strecke2[j]],x3,y3)==0)
        {
          KreuzungspunktX[AnzahlDerKreuzungspunkte]=x3;
          KreuzungspunktY[AnzahlDerKreuzungspunkte]=y3;
          AnzahlDerNachbarn[AnzahlDerKreuzungspunkte]=4;
          // die Gerade i hat jetzt einen Kreuzungspunkt mehr
          anzahl=Gerade1[Gerade2[i]];
          anzahl++;
          Gerade1[Gerade2[i]]=anzahl;
          Gerade1[Gerade2[i]+anzahl]=AnzahlDerKreuzungspunkte+AnzahlDerKnoten;
          // die Gerade j hat jetzt einen Kreuzungspunkt mehr
          anzahl=Gerade1[Gerade2[j]];
          anzahl++;
          Gerade1[Gerade2[j]]=anzahl;
          Gerade1[Gerade2[j]+anzahl]=AnzahlDerKreuzungspunkte+AnzahlDerKnoten;
          AnzahlDerKreuzungspunkte++;
        };
      };
    };
  };       	

  // Berechne den Knotengrad
  if(Knotengrad!=NULL) delete[] Knotengrad;
  if((Knotengrad=new int[AnzahlDerKnoten])==0) SpeicherFehler();			// Speicherplatz reservieren
  int summe;
  for(i=0;i<AnzahlDerKnoten;i++)
  {
    summe=0;
    for(j=0;j<AnzahlDerKnoten;j++)
      summe+=Matrix[i][j];
    Knotengrad[i]=summe;
  };
  return 0;
};

template<class T,class S> int Drawing<T,S>::BestimmeNachbarnDerKnoten(void)		// interne Methode
{
  // wird von BerechneNachbarArrays verwendet
  // bestimme die Nachbarn der Knoten
  T i,j;										// Schleifenvariablen
  T* PositionNachbar;									// dynamisches Array fuer die aktuelle Position der Nachbarn der Knoten oder Kreuzungspunkte
  T speicher;	  									// Hilfsvariable
  if((PositionNachbar=new T[AnzahlDerKnoten])==0) SpeicherFehler();			// reservieren von Speicherplatz
  for(i=0;i<AnzahlDerKnoten;i++)
    PositionNachbar[i]=0;								// Werte initialisieren
  for(i=0;i<AnzahlDerKnoten;i++)							// durchlaufe alle Knoten
  {
    for(j=0;j<AnzahlDerKnoten;j++)							// durchlaufe alle Knoten	
    {
      if(Matrix[i][j]==1)								// wenn die zwei Knoten miteinander verbunden sind
      {
        speicher=GeradenNr[i][j];
        if(Gerade1[Gerade2[speicher]]>0)
        {
          // es gibt innere Kreuzungspunkte
          if(Strecke1[speicher]==i)
          {
            Nachbar1[Nachbar2[i]+PositionNachbar[i]]=Gerade1[Gerade2[speicher]+1];
            PositionNachbar[i]++;							// ein weiterer Nachbar wurde gefunden
          }
          else
          {
            Nachbar1[Nachbar2[i]+PositionNachbar[i]]=Gerade1[Gerade2[speicher]+Gerade1[Gerade2[speicher]]];
            PositionNachbar[i]++;							// ein weiterer Nachbar wurde gefunden
          };
        }
        else
        {
          // es gibt keine inneren Kreuzungspunkte
          if(Strecke1[speicher]==i)
          {
            Nachbar1[Nachbar2[i]+PositionNachbar[i]]=Strecke2[speicher];
            PositionNachbar[i]++;							// ein weiterer Nachbar wurde gefunden
          }
          else
          {
            Nachbar1[Nachbar2[i]+PositionNachbar[i]]=Strecke1[speicher];		// ein weiterer Nachbar wurde gefunden
            PositionNachbar[i]++;
          };
        };
      };
    };
  };
  delete[] PositionNachbar;
  return 0;
};

template<class T,class S> int Drawing<T,S>::BestimmeNachbarnDerKreuzungspunkte(void)	// interne Methode
{
  // wird von BerechneNachbarArrays verwendet
  // bestimme die Nachbar der Kreuzungspunkte
  T i,j;										// Schleifenvariablen
  T speicher4,speicher2;								// Hilfsvariablen
  T* PositionNachbar;									// dynamisches Array fuer die aktuelle Position der Nachbarn der Knoten oder Kreuzungspunkte
  T speicher;	  									// Hilfsvariable
  if((PositionNachbar=new T[AnzahlDerKreuzungspunkte])==0) SpeicherFehler();		// reservieren von Speicherplatz
  for(i=0;i<AnzahlDerKreuzungspunkte;i++)
    PositionNachbar[i]=0;								// Werte initialisieren
  for(i=0;i<AnzahlDerKanten;i++)							// durchlaufe alle Kanten
  {
    speicher=Gerade1[Gerade2[i]];    							
    for(j=0;j<speicher;j++)
    {
      // Nr des Kreuzungspunktes, dessen Nachbarn bestimmt werden sollen
      speicher4=Gerade1[Gerade2[i]+j+1];
      speicher2=speicher4-AnzahlDerKnoten;
      if((j==0) && (speicher==1))
      {
        // linker und rechter Nachbar sind Knoten
        // linker Nachbar
        Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke1[i];
        PositionNachbar[speicher2]++;                              			// weiterer Nachbar gefunden
        // rechter Nachbar
        Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke2[i];
        PositionNachbar[speicher2]++;                                                  	// weiterer Nachbar gefunden
      }
      else
      {
        if(j==0)
        {
          // der linke Nachbar ist ein Knoten und der rechte Nachbar ist ein Kreuzungspunkt
          // linker Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke1[i];
          PositionNachbar[speicher2]++;                                    		// weiterer Nachbar gefunden
          // rechter Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j+2];
          PositionNachbar[speicher2]++;                                           	// weiterer Nachbar gefunden	
        };
        if(j==speicher-1)
        {
          // der linke Nachbar ist ein Kreuzungspunkt und der rechte Nachbar ist ein Knoten
          // linker Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j];
          PositionNachbar[speicher2]++;                                    		// weiterer Nachbar gefunden
          // rechter Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke2[i];
          PositionNachbar[speicher2]++;                                               	// weiterer Nachbar gefunden
        };
        if((j!=0) && (j!=speicher-1))
        {
          // der linke und der rechte Nachbar sind Kreuzungspunkte
          // linker Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j];
          PositionNachbar[speicher2]++;                                        		// weiterer Nachbar gefunden
          // rechter Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j+2];
          PositionNachbar[speicher2]++;                                          	// weiterer Nachbar gefunden	
        };
      };
    };
  };

  if(PositionNachbar!=NULL) delete[] PositionNachbar;                                   // Speicher wieder freigeben
  return 0;
};

template<class T,class S> int Drawing<T,S>::preInitialisiereNachbararrays(void)		// interne-Methode
{
  // wird von BerechneNachbararrays verwendet
  // da die Kreuzungspunkte nun bekannt sind, koennen die Nachbararrays vorinitialisiert und angelegt werden
  T z,i;
  if(Nachbar2!=NULL) delete[] Nachbar2;							// sicher ist sicher
  Nachbar2=NULL;
  if((Nachbar2=new T[AnzahlDerKreuzungspunkte+AnzahlDerKnoten])==0) SpeicherFehler();  // reservieren von Speicherplatz fuer Nachbar2	
  z=0;

  for(i=0;i<AnzahlDerKnoten;i++)
    Knotengrad[i]=AnzahlDerKnoten-1;

  for(i=0;i<AnzahlDerKnoten;i++)
  {
    Nachbar2[i]=z;									// wo in Nachbar1 beginnt der Knoten i?
    z+=Knotengrad[i];									// weiterruecken	
  };
  for(i=0;i<AnzahlDerKreuzungspunkte;i++)
  {
    Nachbar2[i+AnzahlDerKnoten]=z;							// wo in Nachbar1 beginnt der Kreuzungspunkt i?
    z+=AnzahlDerNachbarn[i];								// weiterruecken
  };
  z++;
  if(Nachbar1!=NULL) delete[] Nachbar1;							// sicher ist sicher
  Nachbar1=NULL;
  if(( Nachbar1=new T[z])==0) SpeicherFehler();						// reservieren von Speicherplatz fuer Nachbar1
  return 0;
};

template<class T,class S> int Drawing<T,S>::BerechneNachbarArrays_o2(void)		// Berechnung
{
  // Berechnet die Nachbar-Arrays des Kreuzungsgraphen
  // Es wird so ziemlich saemtlicher Speicherplatz gleich nach der Berechnung wieder
  // freigegeben. Die Methode Berechne AnzahlDreiecke ist immer noch aufrufbar.
  // Aber die Methode BerechneAnzahlPolygone kann nicht mehr verwendet werden
  if( (Gerade1==NULL) || (Gerade2==NULL) || (Strecke1==NULL) || (Strecke2==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  spareSpeicher_1();									// brutales Speicherplatzsparen
  preInitialisiereNachbararrays();							// Lege Nachbararrays Nachbar1 und Nachbar2 an	
  BestimmeNachbarnDerKnoten();                                                          // Bestimme die Nachbarn der Knoten
  spareSpeicher_3();		                       					// brutales Speicherplatzsparen
  BestimmeNachbarnDerKreuzungspunkte();                                                 // Bestimme die Nachbarn der Kreuzungspunkte
  spareSpeicher_2();									// brutales Speicherplatzsparen
  return 0;
};

template<class T,class S> int Drawing<T,S>::BerechneNachbarArrays_o1(void)		// Berechnung
{
  // Berechnet die Nachbar-Arrays des Kreuzungsgraphen
  // Es wird nur soviel Speicherplatz freigegeben, dass man noch die
  // Methode BerechneAnzahlPolygone ausfuehren kann
  if( (Gerade1==NULL) || (Gerade2==NULL) || (Strecke1==NULL) || (Strecke2==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  preInitialisiereNachbararrays();							// Lege Nachbararrays Nachbar1 und Nachbar2 an	
  BestimmeNachbarnDerKnoten();                                                          // Bestimme die Nachbarn der Knoten
  spareSpeicher_3();		                       					// brutales Speicherplatzsparen
  BestimmeNachbarnDerKreuzungspunkte();                                                 // Bestimme die Nachbarn der Kreuzungspunkte
  spareSpeicher_2();									// brutales Speicherplatzsparen
  return 0;
};

template<class T,class S> int Drawing<T,S>::BerechneNachbarArrays(void)			// Berechnung
{
  // Berechnet die Nachbar-Arrays des Kreuzungsgraphen
  // Es wird keinerlei Speicherplatz wieder freigegeben, bis das Drawing-Objekt zerstoert wird
  if( (Gerade1==NULL) || (Gerade2==NULL) || (Strecke1==NULL) || (Strecke2==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  preInitialisiereNachbararrays();							// Lege Nachbararrays Nachbar1 und Nachbar2 an	
  BestimmeNachbarnDerKnoten();                                                          // Bestimme die Nachbarn der Knoten
  BestimmeNachbarnDerKreuzungspunkte();                                                 // Bestimme die Nachbarn der Kreuzungspunkte
  return 0;
};

template<class T,class S> int Drawing<T,S>::spareSpeicher_1(void)			// interne Methode
{
  // spart ziemlich brutal Speicherplatz ein, nach dem Benutzen dieser Funktionen kann man
  // im Allgemeinen nicht mehr auf irgendwelche Zwischenergebnisse zugreifen
  // Diese Funktion sollte moeglichst nicht geandert werden
  if(KnotenX!=NULL) delete[] KnotenX;							// brutales Speicherplatzsparen	
  KnotenX=NULL;										// brutales Speicherplatzsparen
  if(KnotenY!=NULL) delete[] KnotenY;                                                 	// brutales Speicherplatzsparen
  KnotenY=NULL;                                                                       	// brutales Speicherplatzsparen
  if(KreuzungspunktX!=NULL) delete[] KreuzungspunktX;                                 	// brutales Speicherplatzsparen
  KreuzungspunktX=NULL;                                                               	// brutales Speicherplatzsparen
  if(KreuzungspunktY!=NULL) delete[] KreuzungspunktY;                                 	// brutales Speicherplatzsparen
  KreuzungspunktY=NULL;									// brutales Speicherplatzsparen
  return 0;
};

template<class T,class S> int Drawing<T,S>::spareSpeicher_3(void)			// interne Methode
{
  // spart ziemlich brutal Speicherplatz ein, nach dem Benutzen dieser Funktionen kann man
  // im Allgemeinen nicht mehr auf irgendwelche Zwischenergebnisse zugreifen
  // Diese Funktion sollte moeglichst nicht geandert werden
  int i;
  if(GeradenNr!=NULL)
  {
    for(i=0;i<AnzahlDerKnoten;i++)
      if(GeradenNr[i]!=NULL)
      {
        delete[] GeradenNr[i];								// brutales Speicherplatzsparen
        GeradenNr[i]=NULL;
      };
    delete[] GeradenNr;									// brutales Speicherplatzsparen
    GeradenNr=NULL;
  };
  return 0;
};

template<class T,class S> int Drawing<T,S>::spareSpeicher_2(void)			// interne Methode
{
  // spart ziemlich brutal Speicherplatz ein, nach dem Benutzen dieser Funktionen kann man
  // im Allgemeinen nicht mehr auf irgendwelche Zwischenergebnisse zugreifen
  // Diese Funktion sollte moeglichst nicht geandert werden
  int i;
  if(GeradenNr!=NULL)
  {
    for(i=0;i<AnzahlDerKnoten;i++)
      if(GeradenNr[i]!=NULL)
      {
        delete[] GeradenNr[i];								// brutales Speicherplatzsparen
        GeradenNr[i]=NULL;
      };
    delete[] GeradenNr;									// brutales Speicherplatzsparen
    GeradenNr=NULL;
  };
  if(Gerade2!=NULL) delete[] Gerade2;							// brutales Speicherplatzsparen
  Gerade2=NULL;                                                                         // brutales Speicherplatzsparen
  if(Gerade1!=NULL) delete[] Gerade1;                                                   // brutales Speicherplatzsparen
  Gerade1=NULL;                                                                         // brutales Speicherplatzsparen
  if(Strecke1!=NULL) delete[] Strecke1;                                                 // brutales Speicherplatzsparen
  Strecke1=NULL;                                                                        // brutales Speicherplatzsparen
  if(Strecke2!=NULL) delete[] Strecke2;                                                 // brutales Speicherplatzsparen
  Strecke2=NULL;                                                                        // brutales Speicherplatzsparen
  if(Matrix!=NULL)
  {
    for(i=0;i<AnzahlDerKnoten;i++)
      if(Matrix[i]!=NULL)
      {
        delete[] Matrix[i];								// brutales Speicherplatzsparen
        Matrix[i]=NULL;
      };
    delete[] Matrix;									// brutales Speicherplatzsparen
    Matrix=NULL;
  };
  return 0;
};

template<class T,class S> int Drawing<T,S>::spareSpeicher_4(void)			// interne Methode
{
  // spart ziemlich brutal Speicherplatz ein, nach dem Benutzen dieser Funktionen kann man
  // im Allgemeinen nicht mehr auf irgendwelche Zwischenergebnisse zugreifen
  // Diese Funktion sollte moeglichst nicht geandert werden
  int i;
  if(Gerade2!=NULL) delete[] Gerade2;							// brutales Speicherplatzsparen
  Gerade2=NULL;                                                                         // brutales Speicherplatzsparen
  if(Gerade1!=NULL) delete[] Gerade1;                                                   // brutales Speicherplatzsparen
  Gerade1=NULL;                                                                         // brutales Speicherplatzsparen
  if(Strecke1!=NULL) delete[] Strecke1;                                                 // brutales Speicherplatzsparen
  Strecke1=NULL;                                                                        // brutales Speicherplatzsparen
  if(Strecke2!=NULL) delete[] Strecke2;                                                 // brutales Speicherplatzsparen
  Strecke2=NULL;                                                                        // brutales Speicherplatzsparen
  if(Matrix!=NULL)
  {
    for(i=0;i<AnzahlDerKnoten;i++)
      if(Matrix[i]!=NULL)
      {
        delete[] Matrix[i];								// brutales Speicherplatzsparen
        Matrix[i]=NULL;
      };
    delete[] Matrix;									// brutales Speicherplatzsparen
    Matrix=NULL;
  };
  return 0;
};

template<class T,class S> int Drawing<T,S>::spareSpeicher_5(void)			// interne Methode
{
  // spart ziemlich brutal Speicherplatz ein, nach dem Benutzen dieser Funktionen kann man
  // im Allgemeinen nicht mehr auf irgendwelche Zwischenergebnisse zugreifen
  // Diese Funktion sollte moeglichst nicht geandert werden
  // freigeben von saemtlichem reserviertem Speicher
  int i;                         							// Schleifenvariable
  if(AnzahlDerPolygone!=NULL) delete[] AnzahlDerPolygone;				// sicher ist sicher
  AnzahlDerPolygone=NULL;
  if(KnotenX!=NULL) delete[] KnotenX;                					// sicher ist sicher
  KnotenX=NULL;
  if(KnotenY!=NULL) delete[] KnotenY;                                                   // sicher ist sicher
  KnotenY=NULL;
  if(Knotengrad!=NULL) delete[] Knotengrad;         					// sicher ist sicher
  Knotengrad=NULL;
  if(Matrix!=NULL)
  {
    for(i=0;i<AnzahlDerKnoten;i++)
      if(Matrix[i]!=NULL)
      {
        delete[] Matrix[i];								// sicher ist sicher
        Matrix[i]=NULL;
      };
    delete[] Matrix;						
    Matrix=NULL;
  };
  if(KreuzungspunktX!=NULL) delete[] KreuzungspunktX;					// sicher ist sicher
  KreuzungspunktX=NULL;
  if(KreuzungspunktY!=NULL) delete[] KreuzungspunktY;					// sicher ist sicher
  KreuzungspunktY=NULL;
  if(Nachbar1!=NULL) delete[] Nachbar1;							// sicher ist sicher
  Nachbar1=NULL;
  if(Nachbar2!=NULL) delete[] Nachbar2;							// sicher ist sicher
  Nachbar2=NULL;
  if(Strecke1!=NULL) delete[] Strecke1;							// sicher ist sicher
  Strecke1=NULL;
  if(Strecke2!=NULL) delete[] Strecke2;							// sicher ist sicher
  Strecke2=NULL;
  if(Gerade1!=NULL) delete[] Gerade1;							// sicher ist sicher
  Gerade1=NULL;
  if(Gerade2!=NULL) delete[] Gerade2;							// sicher ist sicher
  Gerade2=NULL;
  if(AnzahlDerNachbarn!=NULL) delete[] AnzahlDerNachbarn;				// sicher ist sicher
  AnzahlDerNachbarn=NULL;
  if(GeradenNr!=NULL)
  {
    for(i=0;i<AnzahlDerKnoten;i++)
      if(GeradenNr[i]!=NULL)								// sicher ist sicher
      {
        delete[] GeradenNr[i];
        GeradenNr[i]=NULL;
      };
    delete[] GeradenNr;									// sicher ist sicher
    GeradenNr=NULL;
  };
  return 0;
};

template<class T,class S> Drawing<T,S>::~Drawing()					// Destruktor
{
  // freigeben von reserviertem Speicher beim loeschen des Klassenelements
  spareSpeicher_5();									// Das Loeschen ist komplett in dieser Methode definiert
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlDreiecke_o(void)		// Berechnung
{
  // Berechnet die Anzahl der Dreiecke
  // Es wird nicht auf innere Punkte ueberprueft
  // Speicherplatz optimerte Version, d.h. das nach Aufruf dieser Methode
  // saemtliche Informationen verloren sind, so kann man zum Beispiel nicht mehr auf
  // die Koordinaten der Kreuzungspunkte zugreifen !!
  spareSpeicher_4();									// brutales Speicherplatzsparen
  spareSpeicher_1();									// brutales Speicherplatzsparen
  T rueck;
  rueck=BerechneAnzahlDreiecke();							// Aufruf der nicht- speicheroptimerten Version
  spareSpeicher_5();									// brutales Speicherplatzsparen
  return rueck;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlDreiecke_o_allgemein(void)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke
  // Es wird auf innere Punkte ueberprueft
  // Speicherplatz optimerte Version, d.h. das nach Aufruf dieser Methode
  // saemtliche Informationen verloren sind, so kann man zum Beispiel nicht mehr auf
  // die Koordinaten der Kreuzungspunkte zugreifen !!
  spareSpeicher_4();									// brutales Speicherplatzsparen
  spareSpeicher_1();									// brutales Speicherplatzsparen
  T rueck;
  rueck=BerechneAnzahlDreiecke_allgemein();						// Aufruf der nicht- speicheroptimerten Version
  spareSpeicher_5();									// brutales Speicherplatzsparen
  return rueck;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlDreiecke_allgemein(T von=-1,T bis=-1)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke
  // Es wird auf innere Punkte ueberprueft
  if((von<0) || (von>AnzahlDerKnoten+AnzahlDerKreuzungspunkte) || (bis<0) || (bis>AnzahlDerKnoten+AnzahlDerKreuzungspunkte))
  {
    von=0;										// Standardfall
    bis=AnzahlDerKnoten+AnzahlDerKreuzungspunkte;					// Standardfall
  };
  T AnzahlDerDreiecke=0;
  T i,e,f,g;
  T N[3];
  for(i=von;i<bis;i++) 									// durchlaufe alle Knoten und Kreuzungspunkte (oder die gewuenschten)
  {
    N[0]=i;
    for(e=0;e<IKnotengrad(i)-1;e++)
    {
      N[1]=Nachbar(i,e);    								// ein Nachbar von i
      if(i<N[1])           								// betrachte nur sortierte Tupel	
      {
        for(f=e+1;f<IKnotengrad(i);f++)
        {
          N[2]=Nachbar(i,f);                                                            // ein weiterer BNachbar von i
          if(i<N[2])									// betrachte nur sortierte Tupel
          {		                                      				
            for(g=0;g<IKnotengrad(N[1]);g++) 						// teste ob N1 und N2 benachbart sind
              if(Nachbar(N[1],g)==N[2])
                AnzahlDerDreiecke+=InnererPunkt(N,2,AnzahlDerKnoten);			// die Anzahl der Dreiecke erhoeht sich um eins
          };
        };
      };
    };
  };
  return AnzahlDerDreiecke;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlDreiecke(T von=-1 ,T bis=-1)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke
  // Es wird nicht auf innere Punkte ueberprueft, deswegen sollte man
  // bei Benutzung dieser Methode wissen, dass keine inneren Punkte
  // existieren
  if((von<0) || (von>AnzahlDerKnoten+AnzahlDerKreuzungspunkte) || (bis<0) || (bis>AnzahlDerKnoten+AnzahlDerKreuzungspunkte))
  {
    von=0;										// Standardfall
    bis=AnzahlDerKnoten+AnzahlDerKreuzungspunkte;					// Standardfall
  };
  T AnzahlDerDreiecke=0;
  T i,e,f,g;
  T N1,N2;
  for(i=von;i<bis;i++) 									// durchlaufe alle Knoten und Kreuzungspunkte (oder die gewuenschten)
  {
    for(e=0;e<IKnotengrad(i)-1;e++)
    {
      N1=Nachbar(i,e);    								// ein Nachbar von i
      if(i<N1)           								// betrachte nur sortierte Tupel	
      {
        for(f=e+1;f<IKnotengrad(i);f++)
        {
          N2=Nachbar(i,f);                                                              // ein weiterer BNachbar von i
          if(i<N2)									// betrachte nur sortierte Tupel
          {		                                      				
            for(g=0;g<IKnotengrad(N1);g++) 						// teste ob N1 und N2 benachbart sind
              if(Nachbar(N1,g)==N2)
                AnzahlDerDreiecke++;           						// die Anzahl der Dreiecke erhoeht sich um eins
          };
        };
      };
    };
  };
  return AnzahlDerDreiecke;
};

template<class T,class S> int Drawing<T,S>::InnererPunkt(T* N,int anzahl,T ende=-1)	// interne Methode		
{
  // Die Methode gibt 1 zurueck, wenn das uebergebene Polygon keine inneren Punkte
  // mehr enthaelt, und 0 wenn es keine inneren Punkte enthaelt
  // Diese Methode wird bei den Anzahlberechnungen von Polygonen benutzt
  // Der Test basiert auf einer Formel ueber den Inhalt von Polygonen von:
  // Felix Klein: "Elementarmathematik vom hoeheren Standpunkte aus II"
  T i,j,h;										// Schleifenvariablen	
  swap(N[0],N[1]);
  bool ok;
  S Flaeche1,Flaeche2,x1,x2,x3,y1,y2,y3,zuwachs;					// Variablen fuer die Berechnung		
  if((ende<0) || (ende>AnzahlDerKnoten+AnzahlDerKreuzungspunkte)) ende=AnzahlDerKnoten+AnzahlDerKreuzungspunkte;
  for(i=0;i<ende;i++)									// durchlauf alle zu untersuchenden Knoten
  {
    ok=false;
    for(h=0;h<IKnotengrad(i);h++)
      for(j=0;j<=anzahl;j++)
        if(Nachbar(i,h)==N[j]) ok=true;							// ein innerer Punkt sollte mindestens mit einem Punkt auf dem Polygonrand verbunden sein
    if(ok==true)
      for(j=0;j<=anzahl;j++)
        if(N[j]==i) ok=false;								// ein innerer Punkt ist kein Randpunkt
    if(ok==true)
    {
      Flaeche1=0;
      Flaeche2=0;
      x1=IKnotenX(i);
      y1=IKnotenY(i);
      for(j=0;j<anzahl;j++)
      {
        x2=IKnotenX(N[j]);
        x3=IKnotenX(N[j+1]);
        y2=IKnotenY(N[j]);
        y3=IKnotenY(N[j+1]);
        zuwachs=(x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1)/2;				// Formel fuer den Flaecheninhalt eines Dreiecks
        Flaeche1+=zuwachs;
        Flaeche2+=abs(zuwachs);
      };
      x2=IKnotenX(N[anzahl]);
      x3=IKnotenX(N[0]);
      y2=IKnotenY(N[anzahl]);
      y3=IKnotenY(N[0]);
      zuwachs=(x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1)/2;					// Formel fuer den Flaecheninhalt eines Dreiecks
      Flaeche1+=zuwachs;
      Flaeche2+=abs(zuwachs);
      Flaeche1=abs(Flaeche1);
      if(abs(Flaeche1-Flaeche2)<FASTNULL*anzahl*100)
      {
        swap(N[0],N[1]);
        return 0;
      };
    };
  };
  swap(N[0],N[1]);
  return 1;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlVierecke(T von=-1,T bis=-1)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke und Vierecke
  // Es erfolgt ein Test auf innere Punkte, deswegen ist diese Methode
  // sehr langsam  !!!
  // Dafuer sind die Ergebnisse dieser Routine fuer jedes beliebige
  // Drawing-Objekt richtig
  if(AnzahlDerPolygone==NULL)								// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((von<0) || (von>AnzahlDerKnoten+AnzahlDerKreuzungspunkte) || (bis<0) || (bis>AnzahlDerKnoten+AnzahlDerKreuzungspunkte))
  {
    von=0;										// Standardfall
    bis=AnzahlDerKnoten+AnzahlDerKreuzungspunkte;					// Standardfall
  };
  int i1,i2,i3,i4;									// Schleifenvariablen
  bool f3,f4;										// Variablen zum Testen von Bedingungen
  T i;
  T N[5];										// Variablen fuer die einzelnen Knoten
  for(i=0;i<AnzahlDerKnoten;i++)
    AnzahlDerPolygone[i]=0;								// Initialiesieren
  for(i=von;i<bis;i++) 									// durchlaufe alle Knoten und Kreuzungspunkte (oder nur die gewuenschten=
  {
    N[0]=i;
    for(i1=0;i1<IKnotengrad(i)-1;i1++)							// durchlaufe ein Paar von Nachbarn von i {N1,N2};
    {
      N[1]=Nachbar(i,i1);    								// ein Nachbar von i
      if(i<N[1])           								// betrachte nur "sortierte" Polygone
      for(i2=i1+1;i2<IKnotengrad(i);i2++)						// durchlaufe ein Paar von Nachbarn von i {N1,N2}
      {
        N[2]=Nachbar(i,i2);                                                              	// ein weiterer Nachbar von i
        if(i<N[2])									// betrachte nur "sortierte" Polygone
        {	
          f3=true;
          for(i3=0;i3<IKnotengrad(N[2]);i3++) 						// teste ob N1 und N2 benachbart sind
          {
            N[3]=Nachbar(N[2],i3);
            if(N[3]==N[1])                                                                  // wenn ja, dann
            {
              AnzahlDerPolygone[2]+=InnererPunkt(N,2,AnzahlDerKnoten);							// hat man ein Dreieck gefunden
              f3=false;									// man braucht hier nicht mehr weiter nach einem Viereck suchen
            };
          };
          if(f3==true)									// wenn man noch weitersuchen muss	
          for(i3=0;i3<IKnotengrad(N[2]);i3++)                				// durchlaufe die Nachbarn von N2
          {
            N[3]=Nachbar(N[2],i3);								// ein achbar von N2
            f4=true;
            for(i4=0;i4<IKnotengrad(N[3]);i4++)						// test ob N3 und i benachvart sind
            {
              N[4]=Nachbar(N[3],i4);
              if(N[4]==i) f4=false;							// wenn ja, dann braucht man nicht mehr weiter nach einem Viereck zu suchen	
            };
            if((i<N[3]) && (f4==true))
            for(i4=0;i4<IKnotengrad(N[3]);i4++)						// teste ob N4 und N1 benachbart sind
            {
              N[4]=Nachbar(N[3],i4);
              if(N[4]==N[1]) AnzahlDerPolygone[3]+=InnererPunkt(N,3);					// wenn ja, dann hat man ein Viereck gefunden
            };
          };
        };
      };
    };
  };
  if(AnzahlDerKnoten>3) return AnzahlDerPolygone[3];
  return -1;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlFuenfecke(T von=-1,T bis=-1)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke, Vierecke und Fuenfecke
  // Es erfolgt ein Test auf innere Punkte, deswegen ist diese Methode
  // sehr langsam  !!!
  // Dafuer sind die Ergebnisse dieser Routine fuer jedes beliebige
  // Drawing-Objekt richtig
  if(AnzahlDerPolygone==NULL)								// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((von<0) || (von>AnzahlDerKnoten+AnzahlDerKreuzungspunkte) || (bis<0) || (bis>AnzahlDerKnoten+AnzahlDerKreuzungspunkte))
  {
    von=0;										// Standardfall
    bis=AnzahlDerKnoten+AnzahlDerKreuzungspunkte;					// Standardfall
  };
  int i1,i2,i3,i4,i5;									// Schleifenvariablen
  bool f3,f4,f5;									// Variablen zum Testen von Bedingungen
  T i;
  T N[6];										// Variablen fuer die einzelnen Knoten
  for(i=0;i<AnzahlDerKnoten;i++)
    AnzahlDerPolygone[i]=0;								// Initialiesieren
  for(i=von;i<bis;i++) 									// durchlaufe alle Knoten und Kreuzungspunkte (oder nur die gewuenschten)
  {
    N[0]=i;
    for(i1=0;i1<IKnotengrad(i)-1;i1++)							// durchlaufe ein Paar von Nachbarn von i {N1,N2};
    {
      N[1]=Nachbar(i,i1);    								// ein Nachbar von i
      if(i<N[1])           								// betrachte nur "sortierte" Polygone
      for(i2=i1+1;i2<IKnotengrad(i);i2++)						// durchlaufe ein Paar von Nachbarn von i {N1,N2}
      {
        N[2]=Nachbar(i,i2);                                                             // ein weiterer Nachbar von i
        if(i<N[2])									// betrachte nur "sortierte" Polygone
        {	
          f3=true;
          for(i3=0;i3<IKnotengrad(N[2]);i3++) 						// teste ob N1 und N2 benachbart sind
          {
            N[3]=Nachbar(N[2],i3);
            if(N[3]==N[1])                                                              // wenn ja, dann
            {
              AnzahlDerPolygone[2]+=InnererPunkt(N,2,AnzahlDerKnoten);			// hat man ein Dreieck gefunden
              f3=false;									// man braucht hier nicht mehr weiter nach einem Viereck suchen
            };
          };
          f4=false;
          if(f3==true)									// wenn man noch weitersuchen muss	
          for(i3=0;i3<IKnotengrad(N[2]);i3++)                				// durchlaufe die Nachbarn von N2
          {
            N[3]=Nachbar(N[2],i3);							// ein Nachbar von N2
            f4=true;
            for(i4=0;i4<IKnotengrad(N[3]);i4++)						// teste ob N3 und i benachvart sind
            {
              N[4]=Nachbar(N[3],i4);							
              if(N[4]==i) f4=false;							// wenn ja, dann braucht man nicht mehr weiter nach einem Viereck zu suchen	
            };
            if((i<N[3]) && (f4==true))
            for(i4=0;i4<IKnotengrad(N[3]);i4++)						// teste ob N3 und N1 benachbart sind
            {
              N[4]=Nachbar(N[3],i4);
              if(N[4]==N[1])								// wenn ja, dann
              {
                AnzahlDerPolygone[3]+=InnererPunkt(N,3);		                // hat man ein Viereck gefunden
                f4=false;                                                               // und man braucht nicht mehr weiter nach einem Fuenfeck zu suchen
              };
            };
            f5=false;
            if(f4==true)
            for(i4=0;i4<IKnotengrad(N[3]);i4++)						// durchlauf alle Nachbarn von N3
            {
              N[4]=Nachbar(N[3],i4);							// ein Nachbar von N3
              f5=true;
              for(i5=0;i5<IKnotengrad(N[4]);i5++)					// teste ob N4 mit i oder N2 benachvart ist
              {
                N[5]=Nachbar(N[4],i5);							
                if((N[5]==i) || (N[5]==N[2])) f5=false;					// wenn ja, dann braucht man nicht mehr weiter nach einem Fuenfeck zu suchen	
              };
              if((i<N[4]) && (f5==true))
              for(i5=0;i5<IKnotengrad(N[4]);i5++)					// teste ob N5 und N1 benachbart sind
              {
                N[5]=Nachbar(N[4],i5);
                if(N[5]==N[1])								// wenn ja, dann
                  AnzahlDerPolygone[4]+=InnererPunkt(N,4);		                // hat man ein Fuenfeck gefunden
              };
            };
          };
        };
      };
    };
  };
  if(AnzahlDerKnoten>4) return AnzahlDerPolygone[4];
  return -1;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlDreiecke_o_symmetrisch(void)	// Berechnung	
{
  // Berechnet die Anzahl der Dreicke eines n-ten Teils eines regelmaessigen n-Ecks
  // d.h, dass die Symmetrie ausgenutzt wird
  // Speicherplatz optimerte Version, d.h. das nach Aufruf dieser Methode
  // saemtliche Informationen verloren sind, so kann man zum Beispiel nicht mehr auf
  // die Koordinaten der Kreuzungspunkte zugreifen !!
  spareSpeicher_4();									// brutales Speicherplatzsparen
  spareSpeicher_1();									// brutales Speicherplatzsparen
  T rueck;
  rueck=BerechneAnzahlDreiecke(0,2);							// Aufruf der nicht- speicheroptimerten Version
  rueck+=BerechneAnzahlDreiecke(AnzahlDerKnoten,AnzahlDerKnoten+AnzahlDerKreuzungspunkte);
  spareSpeicher_5();									// brutales Speicherplatzsparen
  return rueck;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlDreiecke_symmetrisch(void)	// Berechnung	
{
  // Berechnet die Anzahl der Dreicke eines n-ten Teils eines regelmaessigen n-Ecks
  // d.h, dass die Symmetrie ausgenutzt wird
  T r=0;
  r=BerechneAnzahlDreiecke(0,2);							// Benutze die allgemeinere Berechnung der Anzahl der Dreiecke	
  r+=BerechneAnzahlDreiecke(AnzahlDerKnoten,AnzahlDerKnoten+AnzahlDerKreuzungspunkte);	// Benutze die allgemeinere Berechnung der Anzahl der Dreiecke
  return r;
};

template<class T,class S> int Drawing<T,S>::BestimmeNachbarn_symmetrisch(void)		// interne Methode
{
  // Bestimmt die Nachbarn der Knoten und der Kreuzungspunkte fuer die
  // symmetrische Berechnung
  // wird von BerechneNachbarArrays_symmetrisch verwendet
  //BestimmeNachbarnDerKnoten();
  T i,j;										// Schleifenvariablen
  T* PositionNachbar;									// dynamisches Array fuer die aktuelle Position der Nachbarn der Knoten oder Kreuzungspunkte
  T speicher;	  									// Hilfsvariable
  if((PositionNachbar=new T[2])==0) SpeicherFehler();					// reservieren von Speicherplatz
  PositionNachbar[0]=0;
  PositionNachbar[1]=0;
  for(i=0;i<AnzahlDerKanten;i++)
  {
    if(Strecke1[i]<2)
    {
      if(Gerade1[Gerade2[i]]>0)
      {
        // es gibt innere Schnittpunkte
        Nachbar1[Nachbar2[Strecke1[i]]+PositionNachbar[Strecke1[i]]]=Gerade1[Gerade2[i]+1];
        PositionNachbar[Strecke1[i]]++;
      }
      else
      {
        // es gibt keine inneren Schnittpunkte
        if(Strecke2[i]<2)
        {
          Nachbar1[Nachbar2[Strecke1[i]]+PositionNachbar[Strecke1[i]]]=Strecke2[i];
          PositionNachbar[Strecke1[i]]++;
        };
      };
    };
    if(Strecke2[i]<2)
    {
      if(Gerade1[Gerade2[i]]>0)
      {
        // es gibt innere Schnittpunkte
        Nachbar1[Nachbar2[Strecke2[i]]+PositionNachbar[Strecke2[i]]]=Gerade1[Gerade2[i]+Gerade1[Gerade2[i]]];
        PositionNachbar[Strecke2[i]]++;
      }
      else
      {
        // es gibt keine inneren Schnittpunkte
        if(Strecke1[i]<2)
        {
          Nachbar1[Nachbar2[Strecke2[i]]+PositionNachbar[Strecke2[i]]]=Strecke1[i];
          PositionNachbar[Strecke2[i]]++;
        };
      };
    };
  };
  for(i=2;i<AnzahlDerKnoten;i++)
    Knotengrad[i]=0;
  for(i=0;i<2;i++)
    Knotengrad[i]=PositionNachbar[i];
  delete[] PositionNachbar;
  // Bestimme Nachbarn der Kreuzungspunkte
  T speicher4,speicher2;								// Hilfsvariablen
  if((PositionNachbar=new T[AnzahlDerKreuzungspunkte])==0) SpeicherFehler();		// reservieren von Speicherplatz
  for(i=0;i<AnzahlDerKreuzungspunkte;i++)
    PositionNachbar[i]=0;								// Werte initialisieren
  for(i=0;i<AnzahlDerKanten;i++)							// durchlaufe alle Kanten
  {
    speicher=Gerade1[Gerade2[i]];    							
    for(j=0;j<speicher;j++)
    {
      // Nr des Kreuzungspunktes, dessen Nachbarn bestimmt werden sollen
      speicher4=Gerade1[Gerade2[i]+j+1];
      speicher2=speicher4-AnzahlDerKnoten;
      if((j==0) && (speicher==1))
      {
        // linker und rechter Nachbar sind Knoten
        // linker Nachbar
        if(Strecke1[i]<2)
        {
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke1[i];
          PositionNachbar[speicher2]++;                              			// weiterer Nachbar gefunden
        };
        // rechter Nachbar
        if(Strecke2[i]<2)
        {
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke2[i];
          PositionNachbar[speicher2]++;                                                 // weiterer Nachbar gefunden
        };
      }
      else
      {
        if(j==0)
        {
          // der linke Nachbar ist ein Knoten und der rechte Nachbar ist ein Kreuzungspunkt
          // linker Nachbar
          if(Strecke1[i]<2)
          {
            Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke1[i];
            PositionNachbar[speicher2]++;                                    		// weiterer Nachbar gefunden
          };
          // rechter Nachbar
            Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j+2];
            PositionNachbar[speicher2]++;                                           	// weiterer Nachbar gefunden	
        };
        if(j==speicher-1)
        {
          // der linke Nachbar ist ein Kreuzungspunkt und der rechte Nachbar ist ein Knoten
          // linker Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j];
          PositionNachbar[speicher2]++;                                    		// weiterer Nachbar gefunden
          // rechter Nachbar
          if(Strecke2[i]<2)
          {
            Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Strecke2[i];
            PositionNachbar[speicher2]++;                                               // weiterer Nachbar gefunden
          };
        };
        if((j!=0) && (j!=speicher-1))
        {
          // der linke und der rechte Nachbar sind Kreuzungspunkte
          // linker Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j];
          PositionNachbar[speicher2]++;                                        		// weiterer Nachbar gefunden
          // rechter Nachbar
          Nachbar1[Nachbar2[speicher4]+PositionNachbar[speicher2]]=Gerade1[Gerade2[i]+j+2];
          PositionNachbar[speicher2]++;                                          	// weiterer Nachbar gefunden	
        };
      };
    };
  };
  for(i=0;i<AnzahlDerKreuzungspunkte;i++)
    AnzahlDerNachbarn[i]=PositionNachbar[i];
  if(PositionNachbar!=NULL) delete[] PositionNachbar;                                   // Speicher wieder freigeben
  return 0;
};

template<class T,class S> int Drawing<T,S>::BerechneNachbarArrays_o1_symmetrisch(void)	// Berechnung
{
  // Berechnet die Nachbar-Arrays des Kreuzungsgraphen
  // Zur symmetrischen Berechnung
  // Es wird nur soviel Speicherplatz freigegeben, dass man noch die
  // Methode BerechneAnzahlPolygone ausfuehren kann
  if( (Gerade1==NULL) || (Gerade2==NULL) || (Strecke1==NULL) || (Strecke2==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  preInitialisiereNachbararrays();							// Lege Nachbararrays Nachbar1 und Nachbar2 an	
  BestimmeNachbarn_symmetrisch();                                                       // Bestimme die Nachbarn der Knoten und der Kreuzungspunkte
  spareSpeicher_3();		                       					// brutales Speicherplatzsparen
  spareSpeicher_2();									// brutales Speicherplatzsparen
  return 0;
};

template<class T,class S> int Drawing<T,S>::BerechneNachbarArrays_symmetrisch(void)	// Berechnung
{
  // Berechnet die Nachbar-Arrays des Kreuzungsgraphen
  // Zur symmetrischen Berechnung
  // Es wird keinerlei Speicherplatz wieder freigegeben, bis das Drawing-Objekt zerstoert wird
  if( (Gerade1==NULL) || (Gerade2==NULL) || (Strecke1==NULL) || (Strecke2==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  preInitialisiereNachbararrays();							// Lege Nachbararrays Nachbar1 und Nachbar2 an	
  BestimmeNachbarn_symmetrisch();                                                       // Bestimme die Nachbarn der Knoten
  return 0;
};

template<class T,class S> int Drawing<T,S>::BerechneNachbarArrays_o2_symmetrisch(void)	// Berechnung	
{
  // Berechnet die Nachbar-Arrays des Kreuzungsgraphen
  // Zur symmetrischen Berechnung
  // Es wird so ziemlich saemtlicher Speicherplatz gleich nach der Berechnung wieder
  // freigegeben. Die Methode Berechne AnzahlDreiecke ist immer noch aufrufbar.
  // Aber die Methode BerechneAnzahlPolygone kann nicht mehr verwendet werden
  if( (Gerade1==NULL) || (Gerade2==NULL) || (Strecke1==NULL) || (Strecke2==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  spareSpeicher_1();									// brutales Speicherplatzsparen
  preInitialisiereNachbararrays(); 							// Lege NachbarArrays Nachbar1 und Nachbar2 an
  BestimmeNachbarn_symmetrisch();							// Bestimmt die Nachbarn der Knoten und der Kreuzungspunkte
  spareSpeicher_3();		                       					// brutales Speicherplatzsparen
  spareSpeicher_2();
  return 0;
};

template<class T,class S> int Drawing<T,S>::init(int n)					// Initialisierungs-Methode
{
  // Initialisiere die Werte und Arrays der Klasse Drawing mit sinnvollen Werten
  int i,j;	
  AnzahlDerKnoten=n;
  if((AnzahlDerPolygone=new T[AnzahlDerKnoten])==0) SpeicherFehler();			// Speicherplatz reservieren
  if((KnotenX=new S[AnzahlDerKnoten])==0) SpeicherFehler();				// Speicherplatz reservieren
  if((KnotenY=new S[AnzahlDerKnoten])==0) SpeicherFehler();     	                // Speicherplatz reservieren
  if((Knotengrad=new int[AnzahlDerKnoten])==0) SpeicherFehler();                        // Speicherplatz reservieren
  for(i=0;i<AnzahlDerKnoten;i++)
  {
    KnotenX[i]=0;                        						// Werte belegen	
    KnotenY[i]=0;                                                                       // Werte belegen
    Knotengrad[i]=0;                                                                    // Werte belegen
  };
  if((Matrix=new int*[AnzahlDerKnoten])==0) SpeicherFehler();         			// Speicherplatz reservieren
  for(i=0;i<AnzahlDerKnoten;i++)
    if((Matrix[i]=new int[AnzahlDerKnoten])==0) SpeicherFehler();                       // Speicherplatz reservieren
  for(i=0;i<AnzahlDerKnoten;i++)
    for(j=0;j<AnzahlDerKnoten;j++)
      Matrix[i][j]=0;               							// Werte belegen
  if((KreuzungspunktX=new S[1])==0) SpeicherFehler();   	                        // mimimal erzeugen
  if((KreuzungspunktY=new S[1])==0) SpeicherFehler(); 	          	                // minimal erzeugen
  if((Nachbar1=new T[1])==0) SpeicherFehler();                                          // minimal erzeugen
  if((Nachbar2=new T[1])==0) SpeicherFehler();                                          // minimal erzeugen
  if((Strecke1=new T[1])==0) SpeicherFehler();                                          // minimal erzeugen
  if((Strecke2=new T[1])==0) SpeicherFehler();                                          // minimal erzeugen
  if((Gerade1=new T[1])==0) SpeicherFehler();                                           // minimal erzeugen
  if((Gerade2=new T[1])==0) SpeicherFehler();                                           // minimal erzeugen
  if((AnzahlDerNachbarn=new int[1])==0) SpeicherFehler();                               // minimal erzeugen
  if((GeradenNr=new T*[AnzahlDerKnoten])==0) SpeicherFehler();				// Speicherplatz reservieren
  for(i=0;i<AnzahlDerKnoten;i++)
    if((GeradenNr[i]=new T[AnzahlDerKnoten])==0) SpeicherFehler();    			// Speicherplatz reservieren
  for(i=0;i<AnzahlDerKnoten;i++)
    for(j=0;j<AnzahlDerKnoten;j++)
      GeradenNr[i][j]=0;                                 				// Werte belegen
  AnzahlDerKreuzungspunkte=0;                                                           // Werte belegen
  AnzahlDerKanten=0;                                                                    // Werte belegen
  AnzahlDerFlaechen=0;									// Werte belegen
  return 0;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon1_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // Speicherplatz optimierte Version
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.BerechneNachbarArrays_o2_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke_o_symmetrisch();					// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  if(n>3) rueck*=n; 									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon2_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // nicht Speicherplatz optimierte Version
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.BerechneNachbarArrays_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke_symmetrisch();						// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  if(n>3) rueck*=n;									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon3_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte      ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_o2_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke_o_symmetrisch();					// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  if(n>3) rueck*=n;									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke  ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon4_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // nicht Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann keine Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  if((n%2)==0)
  {
    cout << "Die aufgerufene Methode kann keine Mehrfachschnittpunkte behandeln." << endl;
    cout << "Rufen Sie bitte eine andere Methode auf." << endl;
    return -1;
  };
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke_symmetrisch();						// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  if(n>3) rueck*=n;									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke:   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon5_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // Speicherplatz optimierte Version
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  d.BerechneNachbarArrays_o2_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke_o_symmetrisch();					// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  if(n>3) rueck*=n;									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon6_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // nicht Speicherplatz optimierte Version
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  d.BerechneNachbarArrays_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  rueck=d.BerechneAnzahlDreiecke_symmetrisch();						// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  if(n>3) rueck*=n;									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon7_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte      ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Verschmelzen                       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_o2_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke_o_symmetrisch();					// Berechnet die Anzahl der Dreiecke mit einer Speicherplatz optimierten Variante
  if(n>3) rueck*=n;									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke  ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> T Drawing<T,S>::AnzahlDreieckeRegularPolygon8_s(int n)	// Berechnung
{
  // Berechnet die Anzahl der Dreiecke eines regulaeren Polygons aus n Ecken (symmetrisch)
  // nicht Speicherplatz optimierte Version
  // gibt Rechenzeiten aus
  // Version zieht Vorteil aus der Symmetrie eines regulaerren n-Ecks
  // Version kann Mehrfachschnittpunkte behandeln
  T rueck;
  if(n<=0) n=1;
  time_t TimeGes,TimeDiff,TimeGess;							// Variablen zur Messung der benoetigten Realzeit	
  time(&TimeGess);									
  time(&TimeGes);
  TimeDiff=TimeGes;
  Drawing<T,S> d(n);									// legt ein Drawing-Objekt an
  d.CreateRegularNGon();                                                                // Erzeugt ein regelmaessiges n-Eck
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Anlegen eines regelmaessigen " << n << "-Ecks:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneKreuzungspunkte_symmetrisch();						// Berechnet die Kreuzungspunkte	
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Kreuzungspunkte:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.sortiereGeradenQS();								// sortiert die Geraden mit Quicksort
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Sortieren der Geraden mit Quicksort";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.VerschmelzeMehrfachSchnittpunkte();							// Beruecksichtigen von Schnittpunkten aus mehr als zwei Geraden
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Verschmelzen                       ";
  if(n>9) cout << " ";
  if(n>99) cout << " ";
  cout << ":   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  d.BerechneNachbarArrays_symmetrisch();						// Berechnet die Nachbar-Arrays mir einer nicht Speicherplatz optimierten Variante
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Nachbar-Arrays:   " << TimeGes-TimeDiff << " sec" << endl;
  TimeDiff=TimeGes;
  rueck=d.BerechneAnzahlDreiecke_symmetrisch();						// Berechnet die Anzahl der Dreiecke mit einer nicht Speicherplatz optimierten Variante
  if(n>3) rueck*=n;									// Symmetrie
  if(n==3) rueck=1;
  cout << "Anzahl der 3-Ecke: " << rueck << endl;
  time(&TimeGes);
  cout << "Benoetigte Zeit fuer das Berechnen der Anzahl der Dreiecke:   " << TimeGes-TimeDiff << " sec" << endl;
  time(&TimeDiff);
  cout << "Benoetigte Gesamtzeit:   " << TimeDiff-TimeGess << " sec" << endl;
  cout << endl;
  return rueck;
};

template<class T,class S> int Drawing<T,S>::VerschmelzeMehrfachSchnittpunkte(void)	// Berechnung
{
  // die Methode sorgt dafuer, dass es bei einem regelmaessigen Polygon Schnittpunkte
  // aus mehr als 2 Geraden geben kann
  if( (KreuzungspunktX==NULL) || (KreuzungspunktY==NULL) || (KnotenX==NULL) || (KnotenY==0) || (Gerade1==NULL) || (Gerade2==NULL) || (Strecke1==0) || (Strecke2==NULL) || (AnzahlDerNachbarn==NULL) || (Knotengrad==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };		
  T i,j,h,anz,a,b,c;									// Hilfsvariablen
  long double m=FASTNULL*100;								// Hoechste Abweichung, welche noch zum Verschmelzen von 2 Punkten fuehrt
  long double abstand;									// Zur Speicherung des Abstandes zweier Punkte
  T* ersetze;										// Feld zur Speicherung von Ersetzungen
  bool hh=true;
  if((ersetze=new T[AnzahlDerKnoten+AnzahlDerKreuzungspunkte])==0) SpeicherFehler();	// Belege das Ersetze-Array mit der Identitaet	
    for(i=0;i<AnzahlDerKnoten+AnzahlDerKreuzungspunkte;i++)
      ersetze[i]=i;
  while(hh==true)									// Solange sich noch etwas geaendert hat
  {
    hh=false;
    for(i=0;i<AnzahlDerKanten;i++)							// durchlaufe alle Kanten
    {
      if(Gerade1[Gerade2[i]]==0)							// wenn die Gerade keine inneren Punkte enthaelt
      {
        abstand=sqrt(pow(IKnotenX(Strecke1[i])-IKnotenX(Strecke2[i]),2)+pow(IKnotenY(Strecke1[i])-IKnotenY(Strecke2[i]),2));
        if(abstand<m)									// wenn verschmolzen werden soll
        {
          a=Strecke1[i];
          b=Strecke2[i];
          c=min(ersetze[a],ersetze[b]);
          ersetze[a]=c;									// merke Ersetzungen
          ersetze[b]=c;                                                                 // merke Ersetzungen
          setIKnotenX(a,(IKnotenX(a)+IKnotenX(c))/2);                           	// Berechne mittlere Koordinate
          setIKnotenY(a,(IKnotenY(a)+IKnotenY(c))/2);				        // Berechne mittlere Koordinate
          setIKnotenX(b,(IKnotenX(b)+IKnotenX(c))/2);                                   // Berechne mittlere Koordinate
          setIKnotenY(b,(IKnotenY(b)+IKnotenY(c))/2);                                   // Berechne mittlere Koordinate
          hh=true;									// es wurde etwas geaendert
        };
      }
      else										// wenn die Gerade innere Punkte enthaelt
      {
        for(j=0;j<Gerade1[Gerade2[i]];j++)
        {
          if(j==0)
          {
            abstand=sqrt(pow(IKnotenX(Strecke1[i])-IKnotenX(Gerade1[Gerade2[i]+j+1]),2)+pow(IKnotenY(Strecke1[i])-IKnotenY(Gerade1[Gerade2[i]+j+1]),2));
            if(abstand<m)								// wenn verschmolzen werden soll
            {
              a=Strecke1[i];
              b=Gerade1[Gerade2[i]+j+1];
              c=min(ersetze[a],ersetze[b]);
              ersetze[a]=c;								// merke Ersetzungen
              ersetze[b]=c;                                                             // merke Ersetzungen
              setIKnotenX(a,(IKnotenX(a)+IKnotenX(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenY(a,(IKnotenY(a)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenX(b,(IKnotenX(b)+IKnotenX(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenY(b,(IKnotenY(b)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              hh=true;									// es wurde etwas geaendert
            };
          }
          else
          {
            abstand=sqrt(pow(IKnotenX(Gerade1[Gerade2[i]+j])-IKnotenX(Gerade1[Gerade2[i]+j+1]),2)+pow(IKnotenY(Gerade1[Gerade2[i]+j])-IKnotenY(Gerade1[Gerade2[i]+j+1]),2));
            if(abstand<m)								// wenn verschmolzen werden soll	
            {
              a=Gerade1[Gerade2[i]+j];
              b=Gerade1[Gerade2[i]+j+1];
              c=min(ersetze[a],ersetze[b]);
              ersetze[a]=c;                                                             // merke Ersetzungen
              ersetze[b]=c;                                                             // merke Ersetzungen
              setIKnotenX(a,(IKnotenX(a)+IKnotenX(c))/2);				// Berechne mittlere Koordinate
              setIKnotenY(a,(IKnotenY(a)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenX(b,(IKnotenX(b)+IKnotenX(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenY(b,(IKnotenY(b)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              hh=true;									// es wurde etwas geaendert
            };
          };
          if(j==Gerade1[Gerade2[i]]-1)
          {
            abstand=sqrt(pow(IKnotenX(Strecke2[i])-IKnotenX(Gerade1[Gerade2[i]+j+1]),2)+pow(IKnotenY(Strecke2[i])-IKnotenY(Gerade1[Gerade2[i]+j+1]),2));
            if(abstand<m)								// wenn verschmolzen werden soll
            {
              a=Strecke2[i];
              b=Gerade1[Gerade2[i]+j+1];
              c=min(ersetze[a],ersetze[b]);
              ersetze[a]=c;								// merke Ersetzungen
              ersetze[b]=c;                                                             // merke Ersetzungen
              setIKnotenX(a,(IKnotenX(a)+IKnotenX(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenY(a,(IKnotenY(a)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenX(b,(IKnotenX(b)+IKnotenX(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenY(b,(IKnotenY(b)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              hh=true;									// es wurde etwas geaendert
            };
          }
          else
          {
            abstand=sqrt(pow(IKnotenX(Gerade1[Gerade2[i]+j+2])-IKnotenX(Gerade1[Gerade2[i]+j+1]),2)+pow(IKnotenY(Gerade1[Gerade2[i]+j+2])-IKnotenY(Gerade1[Gerade2[i]+j+1]),2));
            if(abstand<m)								// wenn verschmolzen werden soll
            {
              a=Gerade1[Gerade2[i]+j+2];
              b=Gerade1[Gerade2[i]+j+1];
              c=min(ersetze[a],ersetze[b]);
              ersetze[a]=c;                                                             // merke Ersetzungen
              ersetze[b]=c;                                       			// merke Ersetzungen
              setIKnotenX(a,(IKnotenX(a)+IKnotenX(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenY(a,(IKnotenY(a)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenX(b,(IKnotenX(b)+IKnotenX(c))/2);                               // Berechne mittlere Koordinate
              setIKnotenY(b,(IKnotenY(b)+IKnotenY(c))/2);                               // Berechne mittlere Koordinate
              hh=true;									// es wurde etwas geaendert
            };
          };
        };
      };
    };
    for(i=0;i<AnzahlDerKanten;i++)							// durchlaufe alle Kanten
    {
      Strecke1[i]=ersetze[Strecke1[i]];							// Ersetze den Anfang der Kante
      Strecke2[i]=ersetze[Strecke2[i]];                                                 // Ersetze das Ende der Kante
      for(j=0;j<Gerade1[Gerade2[i]];j++)						// durchlaufe alle inneren Punkte
        Gerade1[Gerade2[i]+j+1]=ersetze[Gerade1[Gerade2[i]+j+1]];                       // ersetze einen inneren Punkt der Kante
    };
    // Bereinigen von Mehrfachaufzaehlungen von ein und dem selben Kreuzungspunkt
    // auf einer Gerade
    for(i=0;i<AnzahlDerKanten;i++)							// durchlauf alle Kanten
    {
      for(j=0;j<Gerade1[Gerade2[i]]-1;j++)						// durchlaufe alle inneren Punkte
      {
        if(Gerade1[Gerade2[i]+j+1]==Gerade1[Gerade2[i]+j+2])				// wenn der selbe Punkt zweimal aufgezaehlt wird
        {
          Gerade1[Gerade2[i]]--;							// entferne einen Punkt auf der Gerade
          for(h=j+1;h<Gerade1[Gerade2[i]];h++)						// verschiebe entsprechend
            Gerade1[Gerade2[i]+h+1]=Gerade1[Gerade2[i]+h+2];
        };
      };
    };
    // Die Anzahl der Nachbarn stimmen noch nicht ueberein
    for(i=0;i<AnzahlDerKreuzungspunkte;i++)						// initialisieren
      AnzahlDerNachbarn[i]=0;
    for(i=0;i<AnzahlDerKanten;i++)
    {
      for(j=0;j<Gerade1[Gerade2[i]];j++)
        AnzahlDerNachbarn[Gerade1[Gerade2[i]+j+1]-AnzahlDerKnoten]+=2;			// Knoten hat zwei Nachbarn mehr
    };
    anz=0;
    for(i=0;i<AnzahlDerKreuzungspunkte;i++)
      if(AnzahlDerNachbarn[i]>0) anz++;							// Berechne die wirkliche Anzahl der Kreuzungspunkte
    if(anz==AnzahlIntersectionsRegularPolygon(AnzahlDerKnoten))
    {
      hh=false;
      cout << "Anzahl der Kreuzungspunkte: " << anz <<" / " << AnzahlIntersectionsRegularPolygon(AnzahlDerKnoten) <<endl;	// Test
    }
  };
  if(ersetze!=NULL) delete[] ersetze;							// Speicher wieder freigeben	
    ersetze=NULL;
  return 0;
};

template<class T,class S> int Drawing<T,S>::Bedingung(T i,T j)				// interne Methode
{
  // Berechnet, ob zwei Geraden ueberhaupt einen Schnittpunkt im Sektor haben
  T x,y,r,s;										// Hilfsvariablen
  y=Strecke2[i];
  x=Strecke1[i];
  s=Strecke2[j];
  r=Strecke1[j];
  if(x==0) x+=AnzahlDerKnoten;
  if(y==0) y+=AnzahlDerKnoten;
  if(r==0) r+=AnzahlDerKnoten;
  if(s==0) s+=AnzahlDerKnoten;
  if(x<y) swap(x,y);									// sortieren
  if(r<s) swap(r,s);                                                                    // sortieren
  if(x<r)										// sortieren	
  {
    swap(x,r);
    swap(y,s);
  };
  if(x==r) return -1;									// kann nicht schneiden	
  if(y==s) return -1;									// kann nicht schneiden
  if(x==s) return -1;                                                                   // kann nicht schneiden
  if(r==y) return -1;                                                                   // kann nicht schneiden
  if(s>y) return -1;                                                                    // kann nicht schneiden
  if(x<(T)(AnzahlDerKnoten/2)+1) return -1;						// geht vorbei
  if(r<(T)(AnzahlDerKnoten/2)+1) return -1;                                             // geht vorbei
  if(s>(T)(AnzahlDerKnoten/2)) return -1;						// gehr vorbei
  if(y>(T)(AnzahlDerKnoten/2)) return -1;                                               // geht vorbei
  if((x==AnzahlDerKnoten) && (y==(T)(AnzahlDerKnoten/2)))				// Begrenzungslinie
  {
    if(r-s>(T)(AnzahlDerKnoten/2))
      return 0;
    else
      if(r-s==(T)(AnzahlDerKnoten/2))
        if(s==1)
          return 0;
        else
          return -1;
  };
  if((s==1) && (r==(T)(AnzahlDerKnoten/2)+1))						// Begrenzungslinie
  {
    if(x-y>(T)(AnzahlDerKnoten/2))
      return 0;
    else
      if(x-y==(T)(AnzahlDerKnoten/2))
        if(x==AnzahlDerKnoten)
          return 0;
        else
          return -1;

  };
  if(r==(T)(AnzahlDerKnoten/2)+1) return -1;						// ausserhalb
  if(y==(T)(AnzahlDerKnoten/2)) return -1;						// ausserhalb	
  if((x-y<=(T)((AnzahlDerKnoten+1)/2)) && (r-s<=(T)((AnzahlDerKnoten+1)/2))) return -1;	// ausserhalb
  if(x-y<=(T)(AnzahlDerKnoten/2)) return -1;						// ausserhalb	
  if(r-s<=(T)(AnzahlDerKnoten/2)) return -1;						// ausserhalb
  return 0;										// sonst koennte der Schnittpunkt innerhalb liegen
};

template<class T,class S> int Drawing<T,S>::Bedingung2(T i,T j)		  		// interne Methode
{
  // Berechnet ob diese Gerade ueberhaupt durch den Sektor gehen kann (symmetrischer Fall)
  T i2,j2;
  if(i==0) i2=AnzahlDerKnoten; else i2=i;
  if(j==0) j2=AnzahlDerKnoten; else j2=j;
  if(i2<j2) swap(i2,j2);
  if((i2==AnzahlDerKnoten) && (j2==1)) return 0;					// 0-1 ist auf jeden Fall im Sektor
  if(i2<(T)(AnzahlDerKnoten/2)+1) return -1;						// geht aussen vorbei
  if(j2>(T)(AnzahlDerKnoten/2)) return -1;						// geht aussen vorbei
  if(i2-j2<(T)(AnzahlDerKnoten/2)) return -1;						// geht oberhalb vorbei
  if(i2-j2==(T)(AnzahlDerKnoten/2))
  {
    if((i2!=AnzahlDerKnoten) && (j2!=1)) return -1;					// beruehrt nur in einem Punkt
  };
  return 0;
};


template<class T,class S> T Drawing<T,S>::BerechneZaehler_s(T ii)			// interne Methode	
{
  // Berechnet die Anzahl der Schnittpunkte auf dieser Gerade (symmetrischer Fall)
  T i,j;										// Hilfsvariablen
  i=Strecke1[ii];
  j=Strecke2[ii];
  T i2,j2;
  T z=0;
  if(i==0) i2=AnzahlDerKnoten; else i2=i;
  if(j==0) j2=AnzahlDerKnoten; else j2=j;
  if(i2<j2) swap(i2,j2);								// sortieren
  if(i2<(T)(AnzahlDerKnoten/2)+1) return 0;						// es gibt keine Schnittpunkte auf dieser Geraden
  if(j2>(T)(AnzahlDerKnoten/2)) return 0;						// es gibt keine Schnittpunkte auf dieser Geraden
  if(i2-j2<(T)(AnzahlDerKnoten/2)) return 0;						// es gibt keine Schnittpunkte auf dieser Geraden
  if(i2-j2==(T)(AnzahlDerKnoten/2))
  {
    if((i2!=AnzahlDerKnoten) && (j2!=1)) return 0;					// es gibt keine Schnittpunkte auf dieser Geraden
  };
  if((i2==AnzahlDerKnoten) && (j2==1)) return 0;					// es gibt keine Schnittpunkte auf dieser Geraden
  T h,k;
  if((i2==AnzahlDerKnoten) && (j2==(T)(AnzahlDerKnoten/2)))				// Begrenzungslinie
  {
    for(h=(T)(AnzahlDerKnoten/2)+1;h<AnzahlDerKnoten;h++)
      for(k=1;k<(T)(AnzahlDerKnoten/2);k++)
        if(h-k>(T)(AnzahlDerKnoten/2))
          z++;
        else
          if(k==1)
            z++;
          else
            k=(T)(AnzahlDerKnoten/2);
    return z;										// Rueckgabe einer Obergrenze
  };
  if((i2==(T)(AnzahlDerKnoten/2)+1) && (j2==1))						// Begrenzungslinie
  {
    for(h=2;h<=(T)(AnzahlDerKnoten/2);h++)
      for(k=AnzahlDerKnoten;k>(T)(AnzahlDerKnoten/2)+1;k--)
        if(k-h>(T)(AnzahlDerKnoten/2))
          z++;
        else
          if(k==AnzahlDerKnoten)
            z++;
          else
            k=(T)(AnzahlDerKnoten/2);
    return z;										// Rueckgabe einer Obergrenze
  };
  for(h=(T)(AnzahlDerKnoten/2)+2;h<i2;h++)						// keine Begrenzungslinie
    for(k=1;k<j2;k++)
    {
      if(h-k>(T)(AnzahlDerKnoten/2))
        z++;
      else
        k=j2+1;
      if((i2-j2==(T)((AnzahlDerKnoten+1)/2)) && (h-k==(T)((AnzahlDerKnoten+1)/2))) z--;
     };
  for(h=i2+1;h<=AnzahlDerKnoten;h++)
    for(k=j2+1;k<(T)(AnzahlDerKnoten/2);k++)
    {
      if(h-k>(T)(AnzahlDerKnoten/2))
        z++;
      else
        k=(T)(AnzahlDerKnoten/2)+1;
     if((i2-j2==(T)((AnzahlDerKnoten+1)/2)) && (h-k==(T)((AnzahlDerKnoten+1)/2))) z--;
    };
  if(j2!=1) z++;									// Korrektur
  if(i2!=AnzahlDerKnoten) z++;                                                          // Korrektur
  return z;										// Rueckgabe einer Obergrenze	
};

template<class T,class S> int Drawing<T,S>::BerechneKreuzungspunkte_symmetrisch(void)	// Berechnung
{
  // Berechnet die Kreuzungspunkt eines symmetrischen Sektors eines regelmaessigen n-Ecks
  if( (KnotenX==NULL) || (KnotenY==NULL) || (KreuzungspunktX==NULL) || (KreuzungspunktY==NULL) )	// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  AnzahlDerKreuzungspunkte=0;
  double m;										// Abschaetzung der maximalzahl an Kreuzungspunkten	
  long max;
  T i,j,anzahl;
  m=AnzahlDerKnoten*(AnzahlDerKnoten-1)/2;
  m*=(AnzahlDerKnoten-2);
  m/=3;
  m*=(AnzahlDerKnoten-3);
  m/=4;
  m+=AnzahlDerKnoten-1;
  m/=AnzahlDerKnoten;
  m+=AnzahlDerKnoten*AnzahlDerKnoten;
  max=(long)m;										// Ende des Abschaetzens	
  if(Strecke1!=NULL) delete[] Strecke1;							// Sicherheitshalber
  Strecke1=NULL;
  if(Strecke2!=NULL) delete[] Strecke2;                                                	// Sicherheitshalber
  Strecke2=NULL;
  if((Strecke1=new T[AnzahlDerKnoten*(AnzahlDerKnoten-1)/2])==0) SpeicherFehler();  	// Speicherplatz reservieren
  if((Strecke2=new T[AnzahlDerKnoten*(AnzahlDerKnoten-1)/2])==0) SpeicherFehler();	// Speicherplatz reservieren
  AnzahlDerKanten=0;
  for(i=0;i<AnzahlDerKnoten-1;i++)							// Bestimme die moeglichen Kanten
    for(j=i+1;j<AnzahlDerKnoten;j++)
    {
      if(Bedingung2(i,j)==0)								// geht diese Kanten ueberhaupt durch den Sektor??
      {
        Strecke1[AnzahlDerKanten]=i;
        Strecke2[AnzahlDerKanten]=j;
        AnzahlDerKanten++;
      };
    };
  if(Gerade2!=NULL) delete[] Gerade2;							// Sicherheitshalber
  Gerade2=NULL;
  if(Gerade1!=NULL) delete[] Gerade1;                                                	// Sicherheitshalber
  Gerade1=NULL;
  if((Gerade2=new T[AnzahlDerKanten])==0) SpeicherFehler();       			// Speicherplatz reservieren
  if(AnzahlDerNachbarn!=NULL) delete[] AnzahlDerNachbarn;				// Sicherheitshalber
  AnzahlDerNachbarn=NULL;
  if((AnzahlDerNachbarn=new int[max+AnzahlDerKnoten*(AnzahlDerKnoten+1)/2])==0) SpeicherFehler();	// Speicherplatz reservieren	
  j=0;
  for(i=0;i<AnzahlDerKanten;i++)							// Gerade2-Array initialisieren
    Gerade2[i]=0;
  S s1,s2;
  AnzahlDerKreuzungspunkte=0;
  T w[4];
  T h1,h2;
  T zz=0;
  S max_x,max_y,min_x,min_y;								// Hilfsvariablen zum Testen, ob ein Schnittpunkzt innerhalb des Sektors liegt	
  for(i=0;i<AnzahlDerKanten-1;i++)							// durchlaufe alle Paare von Kanten
  {
    for(j=i+1;j<AnzahlDerKanten;j++)                                                    // durchlaufe alle Paare von Kanten
    {
      if(Bedingung(i,j)==0)
      {
        if(Schnittpunktfinden(KnotenX[Strecke1[i]],KnotenY[Strecke1[i]],KnotenX[Strecke2[i]],KnotenY[Strecke2[i]],KnotenX[Strecke1[j]],KnotenY[Strecke1[j]],KnotenX[Strecke2[j]],KnotenY[Strecke2[j]],s1,s2)==0)
        {
          w[0]=Strecke1[i];
          w[1]=Strecke2[i];
          w[2]=Strecke1[j];
          w[3]=Strecke2[j];
          for(h1=0;h1<3;h1++)								// sortieren
            for(h2=h1+1;h2<4;h2++)
              if(w[h1]>w[h2]) swap(w[h1],w[h2]);
          zz=1;
          if(w[0]==0)
          {
            if(w[1]==1) zz=0;
            if(w[2]==(T)(AnzahlDerKnoten/2)) zz=0;
            if(zz==1)
            {
              // testen durch w[1] w[3]
              Schnittpunktfinden(KnotenX[w[1]],KnotenY[w[1]],KnotenX[w[3]],KnotenY[w[3]],KnotenX[0],KnotenY[0],KnotenX[(T)(AnzahlDerKnoten/2)],KnotenY[(T)(AnzahlDerKnoten/2)],max_x,max_y);
              Schnittpunktfinden(KnotenX[w[1]],KnotenY[w[1]],KnotenX[w[3]],KnotenY[w[3]],KnotenX[1],KnotenY[1],KnotenX[(T)(AnzahlDerKnoten/2)+1],KnotenY[(T)(AnzahlDerKnoten/2)+1],min_x,min_y);
              if(abs(max_x-min_x)<100*FASTNULL)
              {
                if(abs(max_y-min_y)<100*FASTNULL)
                {
                  cout << "Genauigkeitsproblem beim symmetrischen berechnen der Kreuzungspunkte" << endl;
                }
                else
                {
                  // unterscheide nach y-Koordinate
                  if(max_y<min_y) swap(max_y,min_y);
                  if((s2-max_y<100*FASTNULL) && (min_y-s2<100*FASTNULL))
                    zz=0;
                };
              }
              else
              {
                // unterscheide nach x-Koordinate
                if(max_x<min_x) swap(max_x,min_x);
                if((s1-max_x<100*FASTNULL) && (min_x-s1<100*FASTNULL))
                  zz=0;
              };
            };
          }
          else
          {
            if((w[1]==1) && (w[3]==(T)(AnzahlDerKnoten/2)+1)) zz=0;
            if((w[0]==1) && (w[2]==(T)(AnzahlDerKnoten/2)+1)) zz=0;
            if(zz==1)
            {
              // testen durch w[0] w[2]
              Schnittpunktfinden(KnotenX[w[0]],KnotenY[w[0]],KnotenX[w[2]],KnotenY[w[2]],KnotenX[0],KnotenY[0],KnotenX[(T)(AnzahlDerKnoten/2)],KnotenY[(T)(AnzahlDerKnoten/2)],max_x,max_y);
              Schnittpunktfinden(KnotenX[w[0]],KnotenY[w[0]],KnotenX[w[2]],KnotenY[w[2]],KnotenX[1],KnotenY[1],KnotenX[(T)(AnzahlDerKnoten/2)+1],KnotenY[(T)(AnzahlDerKnoten/2)+1],min_x,min_y);
              if(abs(max_x-min_x)<100*FASTNULL)
              {
                if(abs(max_y-min_y)<100*FASTNULL)
                {
                  cout << "Genauigkeitsproblem beim symmetrischen berechnen der Kreuzungspunkte" << endl;
                }
                else
                {
                  // unterscheide nach y-Koordinate
                  if(max_y<min_y) swap(max_y,min_y);
                  if((s2-max_y<100*FASTNULL) && (min_y-s2<100*FASTNULL))
                    zz=0;
                };
              }
              else
              {
                // unterscheide nach x-Koordinate
                if(max_x<min_x) swap(max_x,min_x);
                if((s1-max_x<100*FASTNULL) && (min_x-s1<100*FASTNULL))
                  zz=0;
              };
            };
          };
          if(zz==0)
          {
            AnzahlDerKreuzungspunkte++;
            Gerade2[i]++;
            Gerade2[j]++;
          };
        }
        else
        {
          cout << "Fehler beim symmetrischen finden eines Schnittpunktes " << endl;
        };
      };
    };
  };
  if(KreuzungspunktX!=NULL) delete[] KreuzungspunktX;					// Sicherheitshalber
  KreuzungspunktX=NULL;
  if(KreuzungspunktY!=NULL) delete[] KreuzungspunktY;                    		// Sicherheitshalber  		
  KreuzungspunktY=NULL;
  if((KreuzungspunktX=new S[max])==0) SpeicherFehler();           			// Speicherplatz reservieren
  if((KreuzungspunktY=new S[max])==0) SpeicherFehler();                                 // Speicherplatz reservieren
  zz=0;
  for(i=0;i<AnzahlDerKanten;i++)							// Berechne die Groesse von Gerade1
    zz+=Gerade2[i]+1;
  if((Gerade1=new T[zz])==0) SpeicherFehler();       					// Speicherplatz reservieren
  for(i=0;i<zz;i++)									// Gerade1 initialisieren
    Gerade1[i]=0;
  zz=0;
  T zz2;
  for(i=0;i<AnzahlDerKanten;i++)     							// durchlaufe alle Kanten
  {
    zz2=Gerade2[i]+1;
    Gerade2[i]=zz;   									// trage die Positionen im Gerade1-Array ein
    zz+=zz2;
  };
  // Berechne nun wirklich die Schnittpunkte
  AnzahlDerKreuzungspunkte=0;
  for(i=0;i<AnzahlDerKanten-1;i++)							// durchlauf alle Paare von Kanten
  {
    for(j=i+1;j<AnzahlDerKanten;j++)							// Durchlaufe alle Paare von Kanten
    {
      if(Bedingung(i,j)==0)
      {
        if(Schnittpunktfinden(KnotenX[Strecke1[i]],KnotenY[Strecke1[i]],KnotenX[Strecke2[i]],KnotenY[Strecke2[i]],KnotenX[Strecke1[j]],KnotenY[Strecke1[j]],KnotenX[Strecke2[j]],KnotenY[Strecke2[j]],s1,s2)==0)
        {
          w[0]=Strecke1[i];
          w[1]=Strecke2[i];
          w[2]=Strecke1[j];
          w[3]=Strecke2[j];
          for(h1=0;h1<3;h1++)								// sortieren
            for(h2=h1+1;h2<4;h2++)
              if(w[h1]>w[h2]) swap(w[h1],w[h2]);
          zz=1;
          if(w[0]==0)
          {
            if(w[1]==1) zz=0;
            if(w[2]==(T)(AnzahlDerKnoten/2)) zz=0;
            if(zz==1)
            {
              // testen durch w[1] w[3]
              Schnittpunktfinden(KnotenX[w[1]],KnotenY[w[1]],KnotenX[w[3]],KnotenY[w[3]],KnotenX[0],KnotenY[0],KnotenX[(T)(AnzahlDerKnoten/2)],KnotenY[(T)(AnzahlDerKnoten/2)],max_x,max_y);
              Schnittpunktfinden(KnotenX[w[1]],KnotenY[w[1]],KnotenX[w[3]],KnotenY[w[3]],KnotenX[1],KnotenY[1],KnotenX[(T)(AnzahlDerKnoten/2)+1],KnotenY[(T)(AnzahlDerKnoten/2)+1],min_x,min_y);
              if(abs(max_x-min_x)<100*FASTNULL)
              {
                if(abs(max_y-min_y)<100*FASTNULL)
                {
                  cout << "Genauigkeitsproblem beim symmetrischen berechnen der Kreuzungspunkte" << endl;
                }
                else
                {
                  // unterscheide nach y-Koordinate
                  if(max_y<min_y) swap(max_y,min_y);
                  if((s2-max_y<100*FASTNULL) && (min_y-s2<100*FASTNULL))
                    zz=0;
                };
              }
              else
              {
                // unterscheide nach x-Koordinate
                if(max_x<min_x) swap(max_x,min_x);
                if((s1-max_x<100*FASTNULL) && (min_x-s1<100*FASTNULL))
                  zz=0;
               };
            };
          }
          else
          {
            if((w[1]==1) && (w[3]==(T)(AnzahlDerKnoten/2)+1)) zz=0;
            if((w[0]==1) && (w[2]==(T)(AnzahlDerKnoten/2)+1)) zz=0;
            if(zz==1)
            {
              // testen durch w[0] w[2]
              Schnittpunktfinden(KnotenX[w[0]],KnotenY[w[0]],KnotenX[w[2]],KnotenY[w[2]],KnotenX[0],KnotenY[0],KnotenX[(T)(AnzahlDerKnoten/2)],KnotenY[(T)(AnzahlDerKnoten/2)],max_x,max_y);
              Schnittpunktfinden(KnotenX[w[0]],KnotenY[w[0]],KnotenX[w[2]],KnotenY[w[2]],KnotenX[1],KnotenY[1],KnotenX[(T)(AnzahlDerKnoten/2)+1],KnotenY[(T)(AnzahlDerKnoten/2)+1],min_x,min_y);
              if(abs(max_x-min_x)<100*FASTNULL)
              {
                if(abs(max_y-min_y)<100*FASTNULL)
                {
                  cout << "Genauigkeitsproblem beim symmetrischen berechnen der Kreuzungspunkte" << endl;
                }
                else
                {
                  // unterscheide nach y-Koordinate
                  if(max_y<min_y) swap(max_y,min_y);
                  if((s2-max_y<100*FASTNULL) && (min_y-s2<100*FASTNULL))
                    zz=0;
                };
              }
              else
              {
                // unterscheide nach x-Koordinate
                if(max_x<min_x) swap(max_x,min_x);
                if((s1-max_x<100*FASTNULL) && (min_x-s1<100*FASTNULL))
                  zz=0;
              };
            };
          };
          if(zz==0)
          {
            KreuzungspunktX[AnzahlDerKreuzungspunkte]=s1;
            KreuzungspunktY[AnzahlDerKreuzungspunkte]=s2;
            AnzahlDerNachbarn[AnzahlDerKreuzungspunkte]=4;
            // die Gerade i hat jetzt einen Kreuzungspunkt mehr
            anzahl=Gerade1[Gerade2[i]];
            anzahl++;
            Gerade1[Gerade2[i]]=anzahl;
            Gerade1[Gerade2[i]+anzahl]=AnzahlDerKreuzungspunkte+AnzahlDerKnoten;
            // die Gerade j hat jetzt einen Kreuzungspunkt mehr
            anzahl=Gerade1[Gerade2[j]];
            anzahl++;
            Gerade1[Gerade2[j]]=anzahl;
            Gerade1[Gerade2[j]+anzahl]=AnzahlDerKreuzungspunkte+AnzahlDerKnoten;
            AnzahlDerKreuzungspunkte++;
          };
        }
        else
        {
          cout << "Fehler beim symmetrischen finden eines Schnittpunktes " << endl;
        };
      };
    };
  };
  cout << "Anzahl der Kreuzungspunkte: " << AnzahlDerKreuzungspunkte << endl;
  S t;
  t=1.0*AnzahlDerKnoten*AnzahlDerKreuzungspunkte-AnzahlDerKnoten*( (AnzahlDerKnoten-3.0)/2.0+pow( (AnzahlDerKnoten-3.0)/2,2) )/2.0;
  S t2;
  t2=AnzahlDerKnoten*(AnzahlDerKnoten-1)/2.0*(AnzahlDerKnoten-2)/3.0*(AnzahlDerKnoten-3)/4.0;
  if((AnzahlDerKnoten%2)==0)
    cout << "Test leider nicht moeglich." << endl;
  else
    cout << "Test: " << setprecision(10) <<  t << "  <->  " << t2 << endl;
  return 0;
};

template<class T,class S> int Drawing<T,S>::TestAeussereKante(T k1,T k2,T min,T max)	// Interne Methode
{
  // Testet beim einem regelmaessigen n-Eck, ob zwei Knoten eine aeussere Kanten bilden
  if((k1<AnzahlDerKnoten) && (k2<AnzahlDerKnoten)) return 1;				// ja
  return 0;                                                                             // nein
};

template<class T,class S> int Drawing<T,S>::Versuch(T c1,T c2,T cmin,T cmax,int t)	// interne Methode	
{
  // versucht ein Polygon zu umlaufen
  bool maxx,minx;									// Boolvariablen, welche von den 2 moeglichen Umlaufrichtungen ausgefuehrt werden soll
  if(t==0)										// wenn k1 und k2 keine aeussere Kante bilden, dann ueberpruefe beide Umlaufrichtungen	
  {
    maxx=true;
    minx=true;
  }
  else											// ansonsten ergibt nur eine Umlaufrichtung ein Polygon
  {
    if(cmax<AnzahlDerKnoten)								
    {
      maxx=false;
      minx=true;
    }
    else
    {
      maxx=true;
      minx=false;
    };
  };
  if(cmin==cmax) maxx=false;  								// die Umlaufrichtungen muessen sich unterscheiden
  T a1=0;
  T a2=0;
  T a3=0;
  T a4=0;
  long double MaxW,MinW,Winkel,W,d1;							// Hilfsvariablen
  long double x1,x2,x3,y1,y2,y3;							// Hilfsvariablen
  T Min=0;
  T Max=0;
  T n=0;
  bool weiter;										// While-Variable
  int i,v1,v2,v3,zaehl;									// Hilfsvariablen
  if(maxx==true)									// Eine Umlaufrichtung
  {
    a1=c1;										// Knoten des aktuellen Polygonzugs					
    a2=c2;                                                                              // Knoten des aktuellen Polygonzugs
    a3=cmax; 					                                        // Knoten des aktuellen Polygonzugs
    weiter=true;
    zaehl=2;     									// zaehlt die Anzahl der Ecken des Polygons
    while(weiter==true)
    {
      MaxW=0;    									
      MinW=2*PI;
      zaehl++;										// eine Ecke im Polygon mehr
      if(TRICK==1)
      {
        if(zaehl>14) goto sprung4;
      }
      else
      {
        if(zaehl>AnzahlDerKnoten)
        {
          cout << "Genauigkeitsproblem bei der Berechnung der Polygone." << endl;
          goto sprung4;
        };
      };
      for(i=0;i<IKnotengrad(a3);i++)							// durchlauf alle Nachbarn
      {
        n=i;
        a4=Nachbar(a3,n);
        if(a4==c1)
        {
          // Ein neues Polygon wurde gefunden
          goto sprung1;
        };
        if((a4!=a2) && (IKnotengrad(a4)>1))						// wenn man diesem Weg folgen kann ( nicht zurueckgehen, und man kann weiter)
        {
          Winkel=BerechneWinkel(a2,a3,a4);						// den Winkel berechnen
          W=Winkel;
          if(Winkel>MaxW)								// maximalen Winkel bestimmen
          {
            MaxW=Winkel;
            Max=a4;
          };
          if(Winkel<MinW)								// minimalen Winkel bestimmen
          {
            MinW=Winkel;
            Min=a4;
          };
        };
      };
      x1=IKnotenX(a1);								 	
      x2=IKnotenX(a2);
      x3=IKnotenX(a3);
      y1=IKnotenY(a1);
      y2=IKnotenY(a2);
      y3=IKnotenY(a3);
      d1=x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1;    					// doppelten Flaecheninhalt berechnen
      if(d1>0) v1=1; else v1=-1;							// Orientierung bestimmen	
      x1=IKnotenX(a2);
      x2=IKnotenX(a3);
      x3=IKnotenX(Max);
      y1=IKnotenY(a2);
      y2=IKnotenY(a3);
      y3=IKnotenY(Max);
      d1=x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1;						// doppelten Flaecheninhalt berechnen
      if(d1>0) v2=1; else v2=-1;							// Orientierung bestimmen
      x1=IKnotenX(a2);
      x2=IKnotenX(a3);
      x3=IKnotenX(Min);
      y1=IKnotenY(a2);
      y2=IKnotenY(a3);
      y3=IKnotenY(Min);
      d1=x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1;						// doppelten Flaecheninhalt berechnen
      if(d1>0) v3=1; else v3=-1;							// Orientierung bestimmen
      if(v1==v2)									// Wenn der Weg ueber Max konvex ist
      {
        if(v1==v3)									// wenn auch noch gleichzeitig der Weg ueber Min konvex ist
        {
          a4=-1;
          if(abs(BerechneWinkel(a2,a3,Min)-PI)<100*FASTNULL) a4=Max;			// kein Weg mit 180
          if(abs(BerechneWinkel(a2,a3,Max)-PI)<100*FASTNULL) a4=Min;                    // kein Wg mit 180
          if(a4==-1) cout << "fehler 102  #######################################################" << endl;	// fuer einen von den beiden Wegen sollte man sich entscheiden
        }
        else a4=Max;									// Weg ueber Min ist nicht konvex
      }
      else                                      					// der Weg ueber Max ist nicht konvex
      {
        if(v1==v3) a4=Min;								// der Weg ueber Min ist konvex
        else cout << "fehler 101 ###########################################################" << endl;	// keiner von den beiden Wegen ist konvex
      };
      a1=a2;										// verlaengere den aktuellen Polygonzug
      a2=a3;                                                                            // verlaengere den aktuellen Polygonzug
      a3=a4;                                                                            // verlaengere den aktuellen Polygonzug
    };
  };
  goto sprung4;
sprung1:	AnzahlDerPolygone[zaehl-1]++;						// ein zaehl-Eck mehr
sprung4: i++;										// sinnlose Anweisung, nur fuer den Compiler da
  if(minx==true)									// die andere Umlaufrichtung
  {
    a1=c1;										// Knoten des aktuellen Polygonzugs
    a2=c2;                                                                              // Knoten des aktuellen Polygonzugs
    a3=cmin;                                                                            // Knoten des aktuellen Polygonzugs
    weiter=true;
    zaehl=2;            								// zaehlt die Anzahl der Ecken im Polygon
    while(weiter==true)
    {
      MaxW=0;
      MinW=2*PI;
      zaehl++;										// eine Ecke mehr im Polygon
      if(TRICK==1)
      {
        if(zaehl>14) goto sprung3;
      }
      else
      {
        if(zaehl>AnzahlDerKnoten)
        {
          cout << "Genauigkeitsproblem bei der Berechnung der Polygone." << endl;
          goto sprung3;
        };
      };									// eine Ecke mehr im Polygon
      for(i=0;i<IKnotengrad(a3);i++)							// durchlaufe alle Nachbarn
      {
        n=i;
        a4=Nachbar(a3,n);
        if(a4==c1)							
        {
          // Ein neues Polygon wurde gefunden
          goto sprung2;
        };
        if((a4!=a2) && (IKnotengrad(a4)>1))						// wenn man diesem Weg ueberhaupt folgen kann
        {
          Winkel=BerechneWinkel(a2,a3,a4);						// Berechne den Winkel
          W=Winkel;
          if(Winkel>MaxW)								// bestimme maximalen Winkel
          {
            MaxW=Winkel;
            Max=a4;
          };
          if(Winkel<=MinW)								// bestimme minimalen Winkel
          {
            MinW=Winkel;
            Min=a4;
          };
        };
      };
      x1=IKnotenX(a1);
      x2=IKnotenX(a2);
      x3=IKnotenX(a3);
      y1=IKnotenY(a1);
      y2=IKnotenY(a2);
      y3=IKnotenY(a3);
      d1=x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1;						// Berechne doppelten Flaecheninhalt
      if(d1>0) v1=1; else v1=-1;							// Bestimme Orientierung (Umlaufsinn)
      x1=IKnotenX(a2);
      x2=IKnotenX(a3);
      x3=IKnotenX(Max);
      y1=IKnotenY(a2);
      y2=IKnotenY(a3);
      y3=IKnotenY(Max);
      d1=x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1;						// Berechne doppelten Flaecheninhalt
      if(d1>0) v2=1; else v2=-1;                                                        // Bestimme Orientierung (Umlaufsinn)
      x1=IKnotenX(a2);
      x2=IKnotenX(a3);
      x3=IKnotenX(Min);
      y1=IKnotenY(a2);
      y2=IKnotenY(a3);
      y3=IKnotenY(Min);
      d1=x1*y2+x2*y3+x3*y1-y1*x2-y2*x3-y3*x1;						// Berechne doppelten Flaecheninhalt
      if(d1>0) v3=1; else v3=-1;							// Bestimme Orientierung (Umlaufsinn)
      if(v1==v2)									// der Weg ueber Max ist konvex
      {
        if(v1==v3)									// der Weg ueber Min ist auch konvex
        {
          a4=-1;
          if(abs(BerechneWinkel(a2,a3,Min)-PI)<100*FASTNULL) a4=Max;			// keine Wege mit 180		
          if(abs(BerechneWinkel(a2,a3,Max)-PI)<100*FASTNULL) a4=Min;                    // keine Wege mit 180
          if(a4==-1) cout << "fehler 102 #####################################################" << endl; 	// fuer einen Weg muss man sich entscheiden
        }
        else a4=Max;									// Weg ueber Min scheidet aus, da nicht konvex
      }
      else										// Weg ueber Max ist nicht konvex
      {
        if(v1==v3) a4=Min;								// der Weg ueber Min ist konvex
        else cout << "fehler 101 ###################################################################" << endl;    	// keiner der zwei Wege ist konvex
      };
      a1=a2;										// verlaengere den aktuellen Polygonzug
      a2=a3;                                                                            // verlaengere den aktuellen Polygonzug
      a3=a4;                                                                            // verlaengere den aktuellen Polygonzug
    };
  };
  goto sprung3;
sprung2:	AnzahlDerPolygone[zaehl-1]++;						// ein zaehl-Eck mehr
sprung3: i++;
  return 0;
};

template<class T,class S> T Drawing<T,S>::BerechneAnzahlPolygone()			// Berechnung	
{
  // Berechnet die Anzahl der Polygone, also die Anzahl der 3-Ecke,4-Ecke,...
  // Die Methode ist noch nicht allgemeingueltig.
  // - regelmaessige Polygone koennen damit berechnet werden
  // - bei symmetrischen Teilstuecken davon, oder allgemeinen Graphen
  // koennen Fehler auftreten
  T i,e,n,k1,k2,k3;									// Hilfsvariablen	
  long double MaxW,MinW,Winkel,w;							// Hilfsvariablen
  T Max,Min,summe;                                                                      // Hilfsvariablen
  for(i=0;i<AnzahlDerKnoten;i++)
    AnzahlDerPolygone[i]=0;     							// initialisieren
  for(k1=0;k1<AnzahlDerKnoten+AnzahlDerKreuzungspunkte;k1++)				// durchlaufe alle Punkte
  {
    if(IKnotengrad(k1)>1)								// wenn es auf diesem Weg ueberhaupt ein Polygon geben kann
    {
      for(i=0;i<IKnotengrad(k1);i++)							// durchlaufe alle Nachbarn
      {
        n=i;
        k2=Nachbar(k1,n);
        if((IKnotengrad(k2)>1) && (k1<k2))	 					// wenn es auf diesem Weg ueberhaupt ein Polygon geben kann, + sparen der Haelfte der Versuche
        {
          MaxW=0;
          MinW=2*PI;
          Min=-1;
          Max=-1;
          for(e=0;e<IKnotengrad(k2);e++)						// durchlaufe alle Nachbarn
          {
            n=e;
            k3=Nachbar(k2,n);
            if((k1!=k3) && (IKnotengrad(k3)>1))						// wenn es auf diesem Weg ueberhaupt ein Polygon geben kann
            {
              Winkel=BerechneWinkel(k1,k2,k3);						// Berechne den Winkel
              w=Winkel;
              if(Winkel<MinW)								// bestimme minimalen Winkel
              {
                MinW=Winkel;
                Min=k3;
              };
              if(Winkel>MaxW)								// bestimme maximalen Winkel
              {
                MaxW=Winkel;
                Max=k3;
              };
            };
          };
          if((Min>-1) && (Max>-1))							// wenn es auf diesem Weg ueberhaupt ein Polygon geben kann
          {
            Versuch(k1,k2,Min,Max,TestAeussereKante(k1,k2,Min,Max));
          }
          else
          {
            // Tue nichts
          };
        };
      };
    };
  };
  summe=0;
  if(TRICK==1 && AnzahlDerKnoten>14)
  {
    if((AnzahlDerKnoten%2)==1)
      AnzahlDerPolygone[AnzahlDerKnoten-1]=AnzahlDerKnoten;
  };
  for(i=2;i<AnzahlDerKnoten;i++)							// Berechne die Anzahl der Polygone
  {
    AnzahlDerPolygone[i]/=(i+1);
    summe+=AnzahlDerPolygone[i];
    cout << "Anzahl der " << (i+1) << "-Ecke: " << AnzahlDerPolygone[i] << endl;
  };
  return summe;										// Gebe die Anzahl der Polygone zurueck
};



template<class T,class S> S Drawing<T,S>::BerechneWinkel(T k1,T k2,T k3)		// interne Methode
{
  // berechnet den Winkel zwischen drei Knoten bzw. Kreuzungspunkten
  S winkel,a1,a2,b1,b2,d2;								// Hilfsvariablen
  a1=IKnotenX(k1)-IKnotenX(k2);                                                         // verschiebe so, dass der
  a2=IKnotenY(k1)-IKnotenY(k2);                                                         // mittlere Knoten bzw.
  b1=IKnotenX(k3)-IKnotenX(k2);                                                         // Kreuzungspunkt in
  b2=IKnotenY(k3)-IKnotenY(k2);                                                         // (0,0) liegt
  d2=((a1*b1+a2*b2)/sqrt(a1*a1+a2*a2)/sqrt(b1*b1+b2*b2));				// Berechnung von cos(Winkel)
  if(abs(abs(d2)-1)<100*FASTNULL)
  {
    winkel=PI;
    return winkel;
  };
  winkel=acos(d2);      								// Winkel
  if(abs(a1)<FASTNULL)
  {
    // Parallel zur Y-Achse
    if(b1>a1)										// Beruecksichtigung einer Orientierung
    {
      // links, bzw. oben
    }
    else
    {
      // rechts, bzw. unten
      winkel=-winkel+2*PI;
    };
  }
  else
  {
    // Keine Parallele zur Y-Achse
    d2=a2+(b1-a1)*(0-a2)/(0-a1);
    if(b2<d2) 									// Beruecksichtigung einer Orientierung
    {
      // links, bzw. oben
    }
    else
    {
      // rechts, bzw. unten
      winkel=-winkel+2*PI;
    };
  };
  return winkel;
};

template<class T,class S> int Drawing<T,S>::setKnotenX(T i,S x)				// set-Methode
{
  // belegt die X-Koordiante eines Knotens mit einem Wert
  if((i<0) || (i>=AnzahlDerKnoten))							// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht die X-Koordinate eines nicht existierenden Knotens zu belegen." << endl;
    return -1;
  };
  if(KnotenX==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  KnotenX[i]=x;
  return 0;
};

template<class T,class S> int Drawing<T,S>::setKnotenY(T i,S y)				// set-Methode
{
  // belegt die X-Koordiante eines Knotens mit einem Wert
  if((i<0) || (i>=AnzahlDerKnoten))							// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht die Y-Koordinate eines nicht existierenden Knotens zu belegen." << endl;
    return -1;
  };
  if(KnotenY==NULL)									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  KnotenY[i]=y;
  return 0;
};

template<class T,class S> T Drawing<T,S>::getPolygone(int i)				// get-Methode
{
  // Die Methode gibt die Anzahl der i-Ecke zurueck
  if(AnzahlDerPolygone==NULL)								// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht auf ein bereits geloeschtes Array zuzugreifen." << endl;
    return -1;
  };
  if((i<3) || (i>AnzahlDerKnoten))							// test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht die Anzahl der " << i << "-Ecke abzufragen, wobei dies unsinnig ist." << endl;
    return -1;
  };
  return AnzahlDerPolygone[i-1];
};

template<class T,class S> T Drawing<T,S>::NachschlagenDreiecke(int i)			// Nachschlagen von Informationen
{
  // gibt die Anzahl der Dreiecke eines regelmaessigen i-Ecks zurueck,
  // die es im Standardfile "result_dreiecke" sucht
  if(i<0)										// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht nach einem unsinnigen Wert nachzuschlagen." << endl;
    return -1;
  };
  return NachschlagenDreiecke(i,"result_dreiecke");
};

template<class T,class S> T Drawing<T,S>::NachschlagenPolygone(int n,int m)		// Nachschlagen von Informationen
{
  // gibt die Anzahl der m-Ecke eines regelmaessigen n-Ecks zurueck,
  // die es im Standardfile "result_polygone" sucht
  if((m<3) || (m>=AnzahlDerKnoten) || (n<m))						
  {
    return 0;
  };
  return NachschlagenPolygone(n,m,"result_polygone");
};

template<class T,class S> T Drawing<T,S>::NachschlagenPolygone(int n,int m,char* name)	// Nachschlagen von Informationen
{
  // gibt die Anzahl der m-Ecke eines regelmaessigen n-Ecks zurueck,
  // die es im Standardfile "result_polygone" sucht
  if((m<3) || (m>=AnzahlDerKnoten) || (n<m))						
  {
    return 0;
  };
  T rueck=-1;
  T a,b,c;
  bool w=true;
  ifstream file;
  file.open(name,ios::out);
  if(file.fail())									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Das File " << name << " konnte nicht geoeffnet werden." << endl;
    return -1;
  };
  while(w==true)
  {
    if(file.peek()==EOF || file.fail()) break;
    file >> a;
    file >> b;
    file >> c;
    if((a==n) && (b==m))
    {
      rueck=c;
      w=false;
    };
  };
  if(rueck==-1)
    cout << "Die gewuenschte Information konnte leider nicht im file gefunden werden." << endl;
  file.close();
  return rueck;
};

template<class T,class S> T Drawing<T,S>::NachschlagenDreiecke(int i,char* name)	// Nachschlagen von Informationen
{
  // gibt die Anzahl der Dreiecke eines regelmaessigen i-Ecks zurueck,
  // die es im Standardfile "result_dreiecke" sucht
  if(i<0)										// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Es wurde versucht nach einem unsinnigen Wert nachzuschlagen." << endl;
    return -1;
  };
  T rueck=-1;
  T a,c;
  bool w=true;
  ifstream file;
  file.open(name,ios::out);
  if(file.fail())									// Test auf moegliche Fehler
  {
    cout << "Fehler:" << endl;
    cout << "Das File " << name << " konnte nicht geoeffnet werden." << endl;
    return -1;
  };
  while(w==true)
  {
    if(file.peek()==EOF || file.fail()) break;
    file >> a;
    file >> c;
    if(a==i)
    {
      rueck=c;
      w=false;
    };
  };
  if(rueck==-1)
    cout << "Die gewuenschte Information konnte leider nicht im file gefunden werden." << endl;
  file.close();

  return rueck;
};




// Instantierungen

template int swap<long>(long&,long&);							// Instantierung fuer long
template int swap<int>(int&,int&);							// Instantierung fuer int
template int swap<long double>(long double&,long double&); 				// Instantierung fuer long double
template int swap<double>(double&,double&);						// Instantierung fuer double
template int min<int>(int,int);								// Instantierung fuer int
template long min<long>(long,long);							// Instantierung fuer long
template double min<double>(double,double);						// Instantierung fuer double
template long double min<long double>(long double,long double);				// Instantierung fuer long double
template class Drawing<long,long double>;						// Instantierung fuer long, long double 	
template class Drawing<int,long double>;                				// Instantierung fuer long, long double
template class Drawing<long,double>;                    				// Instantierung fuer long, long double
template class Drawing<int,double>;                     				// Instantierung fuer long, long double





